@@ -3,6 +3,7 @@
 extern int STDATTR;
 extern int HIGHATTR;
 extern int BOXATTR;
+extern int ACTIVEATTR;
 extern int BARSTDATTR;
 extern int BARHIGHATTR;
 extern int DESCATTR;
@@ -16,4 +17,5 @@ extern int UDPATTR;
 extern int IGPATTR;
 extern int IGMPATTR;
 extern int IGRPATTR;
+extern int ARPATTR;
 extern int UNKNATTR;
