@@ -18,6 +18,7 @@ details.
 ***/
 
 #include <curses.h>
+#include <stdlib.h>
 #include <panel.h>
 #include <string.h>
 #include "stdwinset.h"
@@ -29,6 +30,7 @@ details.
 int STDATTR;
 int HIGHATTR;
 int BOXATTR;
+int ACTIVEATTR;
 int BARSTDATTR;
 int BARHIGHATTR;
 int DESCATTR;
@@ -41,6 +43,7 @@ int UDPATTR;
 int IGPATTR;
 int IGMPATTR;
 int IGRPATTR;
+int ARPATTR;
 int UNKNATTR;
 
 
@@ -49,18 +52,20 @@ int UNKNATTR;
 void draw_desktop(void)
 {
     int row;			/* counter for desktop construction */
+    char sp_buf[10];
 
     attrset(STDATTR);
     move(0, 0);
-    printw("%80c", 32);		/* these two print the top n' bottom */
+    sprintf(sp_buf, "%%%dc", COLS);
+    printw(sp_buf, ' ');	/* these two print the top n' bottom */
     move(LINES - 1, 0);
-    printw("%80c", 32);		/* lines */
+    printw(sp_buf, ' ');	/* lines */
 
     attrset(FIELDATTR);
 
     for (row = 1; row <= LINES - 2; row++) {	/* draw the background */
 	move(row, 0);
-	printw("%80c", ' ');
+	printw(sp_buf, ' ');
     }
 
     refresh();
@@ -74,7 +79,8 @@ void draw_desktop(void)
 void colorwin(WINDOW * win)
 {
     int ctr;
-    char blankpad[81];
+    char *blankpad;
+    blankpad = (char *) malloc(sizeof(char) * (COLS + 1));
 
     strcpy(blankpad, "");
 
@@ -88,21 +94,24 @@ void colorwin(WINDOW * win)
 	wprintw(win, "%s", blankpad);
     }
     scrollok(win, 1);
+    free(blankpad);
 }
 
 void coloreol()
 {
-    char blankpad[81];
+    char *blankpad;
     int y, x;
     int i;
 
+    blankpad = (char *) malloc(sizeof(char) * (COLS + 1));
     strcpy(blankpad, "");
     getyx(stdscr, y, x);
 
-    for (i = x; i <= 79; i++)
+    for (i = x; i <= COLS - 1; i++)
 	strcat(blankpad, " ");
 
     printw(blankpad);
+    free(blankpad);
 }
 
 void about()
@@ -111,7 +120,7 @@ void about()
     PANEL *panel;
     int ch;
 
-    win = newwin(15, 50, 5, 15);
+    win = newwin(15, 50, (LINES - 15) / 2, (COLS - 50) / 2);
     panel = new_panel(win);
 
     stdwinset(win);
@@ -120,28 +129,18 @@ void about()
     colorwin(win);
     box(win, ACS_VLINE, ACS_HLINE);
     wattrset(win, STDATTR);
-    wmove(win, 1, 2);
-    wprintw(win, "IPTraf");
-    wmove(win, 2, 2);
-    wprintw(win, "An IP Network Statistics Utility");
-    wmove(win, 3, 2);
-    wprintw(win, "Version %s", VERSION);
-    wmove(win, 5, 2);
-    wprintw(win, "Written by Gerard Paul Java");
-    wmove(win, 6, 2);
-    wprintw(win, "Copyright (c) Gerard Paul Java 1997, 1998");
-    wmove(win, 8, 2);
-    wprintw(win, "This program is distributed under the terms of");
-    wmove(win, 9, 2);
-    wprintw(win, "the GNU General Public License Version 2 or");
-    wmove(win, 10, 2);
-    wprintw(win, "any later version. See the included COPYING");
-    wmove(win, 11, 2);
-    wprintw(win, "file for details.");
+    mvwprintw(win, 1, 2, "IPTraf");
+    mvwprintw(win, 2, 2, "An IP Network Statistics Utility");
+    mvwprintw(win, 3, 2, "Version %s", VERSION);
+    mvwprintw(win, 5, 2, "Written by Gerard Paul Java");
+    mvwprintw(win, 6, 2, "Copyright (c) Gerard Paul Java 1997, 1998");
+    mvwprintw(win, 8, 2, "This program is distributed under the terms of");
+    mvwprintw(win, 9, 2, "the GNU General Public License Version 2 or");
+    mvwprintw(win, 10, 2, "any later version. See the included COPYING");
+    mvwprintw(win, 11, 2, "file for details.");
 
     wattrset(win, HIGHATTR);
-    wmove(win, 13, 2);
-    wprintw(win, "Press any key to continue.");
+    mvwprintw(win, 13, 2, ANYKEY_MSG);
 
     update_panels();
     doupdate();
@@ -159,7 +158,7 @@ void masq_warning(void)
     WINDOW *win;
     PANEL *panel;
 
-    win = newwin(11, 50, 7, 15);
+    win = newwin(15, 50, (LINES - 15) / 2, (COLS - 50) / 2);
     panel = new_panel(win);
 
     stdwinset(win);
@@ -169,34 +168,36 @@ void masq_warning(void)
     box(win, ACS_VLINE, ACS_HLINE);
     wattrset(win, ERRTXTATTR);
     mvwprintw(win, 1, 2, "This computer is running with IP Masquerading");
-    mvwprintw(win, 2, 2, "enabled.  The various facilities may give");
-    mvwprintw(win, 3, 2, "strange results because of the translation of");
-    mvwprintw(win, 4, 2, "the packets' IP addresses.  You will get");
-    mvwprintw(win, 5, 2, "better results if you run IPTraf on the");
-    mvwprintw(win, 6, 2, "network segments connected to this machine,");
-    mvwprintw(win, 7, 2, "but not on this machine itself.");
+    mvwprintw(win, 2, 2, "enabled.  If this machine is a masquerading");
+    mvwprintw(win, 3, 2, "router, the various facilities may give");
+    mvwprintw(win, 4, 2, "strange results because of the translation of");
+    mvwprintw(win, 5, 2, "the packets' IP addresses.  You will get");
+    mvwprintw(win, 6, 2, "better results if you run IPTraf on the");
+    mvwprintw(win, 7, 2, "network segments connected to this machine,");
+    mvwprintw(win, 8, 2, "but not on this machine itself.");
+    mvwprintw(win, 10, 2, "If this computer is not a masquerading router");
+    mvwprintw(win, 11, 2, "you can ignore this message.");
     wattrset(win, ERRBOXATTR);
-    mvwprintw(win, 9, 2, ANYKEY_MSG);
+    mvwprintw(win, 13, 2, ANYKEY_MSG);
 
     wgetch(win);
-    
+
     del_panel(panel);
     delwin(win);
-    update_panels(); doupdate();
+    update_panels();
+    doupdate();
 }
 
 void printnomem()
 {
-    move(0, 68);
     attrset(ERRTXTATTR);
-    printw(" Memory Low ");
+    mvprintw(0, 68, " Memory Low ");
 }
 
 void printipcerr()
 {
-    move(0, 68);
     attrset(ERRTXTATTR);
-    printw("  IPC Error ");
+    mvprintw(0, 68, "  IPC Error ");
 }
 
 void printkeyhelp(char *keytext, char *desc, WINDOW * win)
@@ -247,28 +248,28 @@ void tabkeyhelp(WINDOW * win)
 
 void indicate(char *message)
 {
-    move(LINES - 1, 1);
+    char sp_buf[10];
     attrset(STDATTR);
-    printw("%79c", ' ');
-    move(LINES - 1, 1);
-    printw(message);
+    sprintf(sp_buf, "%%%dc", COLS - 1);
+    mvprintw(LINES - 1, 1, sp_buf, ' ');
+    mvprintw(LINES - 1, 1, message);
     refresh();
 }
 
-void printlargenum(unsigned long long i, WINDOW *win)
+void printlargenum(unsigned long long i, WINDOW * win)
 {
-    if (i < 100000000)              /* less than 100 million */
-        wprintw(win, "%9llu", i);
-    else if (i < 1000000000)	   /* less than 1 billion */
-        wprintw(win, "%8lluK", i / 1000);
-    else if (i < 100000000000ULL)     /* less than 100 billion */
-        wprintw(win, "%8lluM", i / 1000000);
-    else if (i < 100000000000000ULL)  /* less than 100 trillion */
-        wprintw(win, "%8lluG", i / 1000000000ULL);
+    if (i < 100000000)		/* less than 100 million */
+	wprintw(win, "%9llu", i);
+    else if (i < 1000000000)	/* less than 1 billion */
+	wprintw(win, "%8lluK", i / 1000);
+    else if (i < 100000000000ULL)	/* less than 100 billion */
+	wprintw(win, "%8lluM", i / 1000000);
+    else if (i < 100000000000000ULL)	/* less than 100 trillion */
+	wprintw(win, "%8lluG", i / 1000000000ULL);
     else
-        wprintw(win, "%8lluT", i / 1000000000000ULL);
+	wprintw(win, "%8lluT", i / 1000000000000ULL);
 }
-        
+
 void standardcolors(int color)
 {
     if ((color) && (has_colors())) {
@@ -291,6 +292,7 @@ void standardcolors(int color)
 	STDATTR = COLOR_PAIR(14) | A_BOLD;
 	HIGHATTR = COLOR_PAIR(3) | A_BOLD;
 	BOXATTR = COLOR_PAIR(3);
+	ACTIVEATTR = COLOR_PAIR(10) | A_BOLD;
 	BARSTDATTR = COLOR_PAIR(15) | A_BOLD;
 	BARHIGHATTR = COLOR_PAIR(11) | A_BOLD;
 	DESCATTR = COLOR_PAIR(2);
@@ -303,11 +305,13 @@ void standardcolors(int color)
 	IGPATTR = COLOR_PAIR(12);
 	IGMPATTR = COLOR_PAIR(10) | A_BOLD;
 	IGRPATTR = COLOR_PAIR(16) | A_BOLD;
+	ARPATTR = COLOR_PAIR(5) | A_BOLD;
 	UNKNATTR = COLOR_PAIR(4) | A_BOLD;
     } else {
 	STDATTR = A_NORMAL;
-	HIGHATTR = A_REVERSE;
+	HIGHATTR = A_BOLD;
 	BOXATTR = A_NORMAL;
+	ACTIVEATTR = A_BOLD;
 	BARSTDATTR = A_REVERSE;
 	BARHIGHATTR = A_BOLD;
 	DESCATTR = A_NORMAL;
@@ -320,6 +324,7 @@ void standardcolors(int color)
 	IGPATTR = A_REVERSE;
 	IGMPATTR = A_REVERSE;
 	IGRPATTR = A_REVERSE;
+	ARPATTR = A_BOLD;
 	UNKNATTR = A_BOLD;
     }
 }
