@@ -28,7 +28,7 @@ void errbox(char *message, char *prompt, int *response)
     WINDOW *win;
     PANEL *panel;
 
-    win = newwin(4, 70, 10, 5);
+    win = newwin(4, 70, (LINES - 4) / 2, (COLS - 70) / 2);
     panel = new_panel(win);
 
     wattrset(win, ERRBOXATTR);
