@@ -1,3 +1,4 @@
+
 /***
 
 fltmgr.c - filter list management routines
@@ -68,6 +69,9 @@ int loadfilterlist(unsigned int protocol, struct ffnode **fltfile)
     close(pfd);
     *fltfile = ffiles;
 
+    if (ffiles == NULL)
+	result = 1;
+
     return result;
 }
 
