@@ -1,4 +1,5 @@
 
+
 /***
 
 hostmon.c - Host traffic monitor
@@ -18,16 +19,14 @@ details.
 
 ***/
 
-#ifndef _I386_TYPES_H
-#include <asm/types.h>
-#endif
-
 #include <curses.h>
 #include <panel.h>
 #include <string.h>
 #include <unistd.h>
+#include <signal.h>
 #include <sys/socket.h>
 #include <sys/types.h>
+#include <asm/types.h>
 #include <netinet/in.h>
 #include <linux/if_packet.h>
 #include <linux/if_ether.h>
@@ -37,6 +36,8 @@ details.
 #include <sys/time.h>
 #include "dirs.h"
 #include "deskman.h"
+#include "links.h"
+#include "isdntab.h"		/* needed by packet.h */
 #include "packet.h"
 #include "hostmon.h"
 #include "stdwinset.h"
@@ -47,6 +48,8 @@ details.
 #include "options.h"
 #include "instances.h"
 #include "error.h"
+#include "mode.h"
+#include "logvars.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
@@ -55,9 +58,20 @@ details.
  * from log.c, applicable only to this module 
  */
 
-extern void writeethlog(struct ethtabent *list, unsigned long nsecs, FILE * logfile);
+extern void writeethlog(struct ethtabent *list, int units, long nsecs, FILE * logfile);
 extern char *ltrim(char *buf);
 
+/*
+ * SIGUSR1 logfile rotation handler
+ */
+
+void rotate_lanlog()
+{
+    rotate_flag = 1;
+    strcpy(target_logname, LANLOG);
+    signal(SIGUSR1, rotate_lanlog);
+}
+
 void ethlook(struct desclist *list, char *address, char *target)
 {
     struct desclistent *ptmp = list->head;
@@ -71,40 +85,45 @@ void ethlook(struct desclist *list, char *address, char *target)
     }
 }
 
-void initethtab(struct ethtab *table)
+void initethtab(struct ethtab *table, int unit)
 {
+    char unitstring[7];
+
     table->head = table->tail = NULL;
     table->firstvisible = table->lastvisible = NULL;
     table->count = table->entcount = 0;
 
-    table->borderwin = newwin(LINES - 2, 80, 1, 0);
+    table->borderwin = newwin(LINES - 2, COLS, 1, 0);
     table->borderpanel = new_panel(table->borderwin);
 
-    table->tabwin = newwin(LINES - 4, 78, 2, 1);
+    table->tabwin = newwin(LINES - 4, COLS - 2, 2, 1);
     table->tabpanel = new_panel(table->tabwin);
 
     wattrset(table->borderwin, BOXATTR);
     box(table->borderwin, ACS_VLINE, ACS_HLINE);
-    wmove(table->borderwin, 0, 5);
+    wmove(table->borderwin, 0, 5 * COLS / 80);
     wprintw(table->borderwin, " PktsIn ");
-    wmove(table->borderwin, 0, 16);
+    wmove(table->borderwin, 0, 16 * COLS / 80);
     wprintw(table->borderwin, " IP In ");
-    wmove(table->borderwin, 0, 24);
+    wmove(table->borderwin, 0, 24 * COLS / 80);
     wprintw(table->borderwin, " BytesIn ");
-    wmove(table->borderwin, 0, 34);
+    wmove(table->borderwin, 0, 34 * COLS / 80);
     wprintw(table->borderwin, " InRate ");
 
-    wmove(table->borderwin, 0, 42);
+    wmove(table->borderwin, 0, 42 * COLS / 80);
     wprintw(table->borderwin, " PktsOut ");
-    wmove(table->borderwin, 0, 53);
+    wmove(table->borderwin, 0, 53 * COLS / 80);
     wprintw(table->borderwin, " IP Out ");
-    wmove(table->borderwin, 0, 61);
+    wmove(table->borderwin, 0, 61 * COLS / 80);
     wprintw(table->borderwin, " BytesOut ");
-    wmove(table->borderwin, 0, 70);
+    wmove(table->borderwin, 0, 70 * COLS / 80);
     wprintw(table->borderwin, " OutRate ");
 
     wmove(table->borderwin, LINES - 3, 40);
-    wprintw(table->borderwin, " InRate and OutRate are in kbits/sec ");
+
+    dispmode(unit, unitstring);
+
+    wprintw(table->borderwin, " InRate and OutRate are in %s/sec ", unitstring);
 
     wattrset(table->tabwin, STDATTR);
     colorwin(table->tabwin);
@@ -150,7 +169,7 @@ struct ethtabent *addethnode(struct ethtab *table, int *nomem)
 void convmacaddr(char *addr, char *result)
 {
     unsigned int i;
-    __u8 *ptmp = addr;
+    u_int8_t *ptmp = addr;
     char hexbyte[3];
 
     strcpy(result, "");
@@ -161,7 +180,7 @@ void convmacaddr(char *addr, char *result)
     }
 }
 
-struct ethtabent *addethentry(struct ethtab *table, unsigned int linktype, 
+struct ethtabent *addethentry(struct ethtab *table, unsigned int linktype,
 			      char *ifname, char *addr, int *nomem,
 			      struct desclist *list)
 {
@@ -179,7 +198,7 @@ struct ethtabent *addethentry(struct ethtab *table, unsigned int linktype,
     ptemp->un.desc.linktype = linktype;
     ethlook(list, ptemp->un.desc.ascaddr, ptemp->un.desc.desc);
     strcpy(ptemp->un.desc.ifname, ifname);
-    
+
     if (strcmp(ptemp->un.desc.desc, "") == 0)
 	ptemp->un.desc.withdesc = 0;
     else
@@ -256,19 +275,21 @@ void printethent(struct ethtab *table, struct ethtabent *entry,
     if (entry->type == 0) {
 	wmove(table->tabwin, target_row, 1);
 	wattrset(table->tabwin, STDATTR);
-	
-	if (entry->un.desc.linktype == ARPHRD_ETHER)
+
+	if (entry->un.desc.linktype == LINK_ETHERNET)
 	    wprintw(table->tabwin, "Ethernet");
-	else if (entry->un.desc.linktype == ARPHRD_FDDI)
+	else if (entry->un.desc.linktype == LINK_PLIP)
+	    wprintw(table->tabwin, "PLIP");
+	else if (entry->un.desc.linktype == LINK_FDDI)
 	    wprintw(table->tabwin, "FDDI");
-	
+
 	wprintw(table->tabwin, " HW addr: %s", entry->un.desc.ascaddr);
 
 	if (entry->un.desc.withdesc)
 	    wprintw(table->tabwin, " (%s)", entry->un.desc.desc);
 
 	wprintw(table->tabwin, " on %s", entry->un.desc.ifname);
-	
+
 	entry->un.desc.printed = 1;
     } else {
 	wattrset(table->tabwin, PTRATTR);
@@ -279,20 +300,20 @@ void printethent(struct ethtab *table, struct ethtabent *entry,
 
 	/* Inbound traffic counts */
 
-	wmove(table->tabwin, target_row, 2);
+	wmove(table->tabwin, target_row, 2 * COLS / 80);
 	printlargenum(entry->un.figs.inpcount, table->tabwin);
-	wmove(table->tabwin, target_row, 12);
+	wmove(table->tabwin, target_row, 12 * COLS / 80);
 	printlargenum(entry->un.figs.inippcount, table->tabwin);
-	wmove(table->tabwin, target_row, 22);
+	wmove(table->tabwin, target_row, 22 * COLS / 80);
 	printlargenum(entry->un.figs.inbcount, table->tabwin);
-	
+
 	/* Outbound traffic counts */
 
-	wmove(table->tabwin, target_row, 40);
+	wmove(table->tabwin, target_row, 40 * COLS / 80);
 	printlargenum(entry->un.figs.outpcount, table->tabwin);
-	wmove(table->tabwin, target_row, 50);
+	wmove(table->tabwin, target_row, 50 * COLS / 80);
 	printlargenum(entry->un.figs.outippcount, table->tabwin);
-	wmove(table->tabwin, target_row, 60);
+	wmove(table->tabwin, target_row, 60 * COLS / 80);
 	printlargenum(entry->un.figs.outbcount, table->tabwin);
     }
 }
@@ -325,14 +346,14 @@ void printrates(struct ethtab *table, unsigned int target_row,
 		struct ethtabent *ptmp)
 {
     if (ptmp->un.figs.past5) {
-	wmove(table->tabwin, target_row, 32);
+	wmove(table->tabwin, target_row, 32 * COLS / 80);
 	wprintw(table->tabwin, "%8.1f", ptmp->un.figs.inrate);
-	wmove(table->tabwin, target_row, 69);
+	wmove(table->tabwin, target_row, 69 * COLS / 80);
 	wprintw(table->tabwin, "%8.1f", ptmp->un.figs.outrate);
     }
 }
 
-void updateethrates(struct ethtab *table, time_t starttime, time_t now,
+void updateethrates(struct ethtab *table, int unit, time_t starttime, time_t now,
 		    unsigned int idx)
 {
     struct ethtabent *ptmp = table->head;
@@ -344,10 +365,17 @@ void updateethrates(struct ethtab *table, time_t starttime, time_t now,
     while (ptmp != NULL) {
 	if (ptmp->type == 1) {
 	    ptmp->un.figs.past5 = 1;
-	    ptmp->un.figs.inrate =
-		((float) (ptmp->un.figs.inspanbr * 8 / 1000)) / ((float) (now - starttime));
-	    ptmp->un.figs.outrate =
-		((float) (ptmp->un.figs.outspanbr * 8 / 1000)) / ((float) (now - starttime));
+	    if (unit == KBITS) {
+		ptmp->un.figs.inrate =
+		    ((float) (ptmp->un.figs.inspanbr * 8 / 1000)) / ((float) (now - starttime));
+		ptmp->un.figs.outrate =
+		    ((float) (ptmp->un.figs.outspanbr * 8 / 1000)) / ((float) (now - starttime));
+	    } else {
+		ptmp->un.figs.inrate =
+		    ((float) (ptmp->un.figs.inspanbr / 1024)) / ((float) (now - starttime));
+		ptmp->un.figs.outrate =
+		    ((float) (ptmp->un.figs.outspanbr / 1024)) / ((float) (now - starttime));
+	    }
 	    if ((ptmp->index >= idx) && (ptmp->index <= idx + LINES - 5)) {
 		wattrset(table->tabwin, HIGHATTR);
 		target_row = ptmp->index - idx;
@@ -361,6 +389,8 @@ void updateethrates(struct ethtab *table, time_t starttime, time_t now,
 
 void scrollethwin(struct ethtab *table, int direction, int *idx)
 {
+    char sp_buf[10];
+    sprintf(sp_buf, "%%%dc", COLS - 2);
     wattrset(table->tabwin, STDATTR);
     if (direction == SCROLLUP) {
 	if (table->lastvisible != table->tail) {
@@ -370,7 +400,7 @@ void scrollethwin(struct ethtab *table, int direction, int *idx)
 	    (*idx)++;
 	    wmove(table->tabwin, LINES - 5, 0);
 	    scrollok(table->tabwin, 0);
-	    wprintw(table->tabwin, "%78c", ' ');
+	    wprintw(table->tabwin, sp_buf, ' ');
 	    scrollok(table->tabwin, 1);
 	    printethent(table, table->lastvisible, *idx);
 	    if (table->lastvisible->type == 1)
@@ -383,7 +413,7 @@ void scrollethwin(struct ethtab *table, int direction, int *idx)
 	    table->firstvisible = table->firstvisible->prev_entry;
 	    (*idx)--;
 	    wmove(table->tabwin, 0, 0);
-	    wprintw(table->tabwin, "%78c", ' ');
+	    wprintw(table->tabwin, sp_buf, ' ');
 	    printethent(table, table->firstvisible, *idx);
 	    if (table->firstvisible->type == 1)
 		printrates(table, 0, table->firstvisible);
@@ -411,14 +441,15 @@ void pageethwin(struct ethtab *table, int direction, int *idx)
 /*
  * The LAN station monitor
  */
- 
+
 void hostmon(const struct OPTIONS *options, int facilitytime)
 {
     int logging = options->logging;
     int fd;
     struct ethtab table;
     struct ethtabent *entry;
-    struct sockaddr_pkt fromaddr;
+    struct sockaddr_ll fromaddr;
+    unsigned short linktype;
 
     int br;
     char buf[8192];
@@ -429,14 +460,18 @@ void hostmon(const struct OPTIONS *options, int facilitytime)
     int ch;
     int endloop = 0;
 
+    char ifname[8];
+
     struct timeval tv;
     unsigned long starttime;
-    unsigned long now = 0, unow = 0;
+    unsigned long now = 0;
+    unsigned long long unow = 0;
     unsigned long statbegin = 0, startlog = 0;
-    unsigned long updtime = 0, updtime_usec = 0;
+    unsigned long updtime = 0;
+    unsigned long long updtime_usec = 0;
 
-    struct desclist elist;             /* Ethernet description list */
-    struct desclist flist;	       /* FDDI description list */
+    struct desclist elist;	/* Ethernet description list */
+    struct desclist flist;	/* FDDI description list */
     struct desclist *list = NULL;
 
     FILE *logfile = NULL;
@@ -444,18 +479,24 @@ void hostmon(const struct OPTIONS *options, int facilitytime)
     int nomem = 0;
 
     if (!facility_active(LANMONIDFILE))
-        mark_facility(LANMONIDFILE, "LAN monitor");
+	mark_facility(LANMONIDFILE, "LAN monitor");
     else {
-        errbox("LAN station monitor already active in another process",
-               ANYKEY_MSG, &ch);
+	errbox("LAN station monitor already active in another process",
+	       ANYKEY_MSG, &ch);
 	return;
-    }        
+    }
 
+    open_socket(&fd);
+
+    if (fd < 0) {
+	unlink(LANMONIDFILE);
+	return;
+    }
     hostmonhelp();
 
-    initethtab(&table);
-    loaddesclist(&elist, ARPHRD_ETHER);
-    loaddesclist(&flist, ARPHRD_FDDI);
+    initethtab(&table, options->actmode);
+    loaddesclist(&elist, LINK_ETHERNET);
+    loaddesclist(&flist, LINK_FDDI);
 
     if (logging) {
 	opentlog(&logfile, LANLOG);
@@ -463,19 +504,18 @@ void hostmon(const struct OPTIONS *options, int facilitytime)
 	if (logfile == NULL)
 	    logging = 0;
     }
-    writelog(logging, logfile, "******** LAN traffic monitor started");
-    open_socket(&fd, "");
+    if (logging)
+	signal(SIGUSR1, rotate_lanlog);
 
-    if (fd < 0)
-	return;
-	
+    rotate_flag = 0;
+    writelog(logging, logfile, "******** LAN traffic monitor started");
     leaveok(table.tabwin, TRUE);
 
     gettimeofday(&tv, NULL);
     starttime = statbegin = startlog = tv.tv_sec;
 
     do {
-	getpacket(fd, buf, &fromaddr, &ch, &br, 5, table.tabwin);
+	getpacket(fd, buf, &fromaddr, &ch, &br, ifname, table.tabwin);
 
 	if (ch != ERR)
 	    switch (ch) {
@@ -497,13 +537,15 @@ void hostmon(const struct OPTIONS *options, int facilitytime)
 	    case 'Q':
 	    case 'x':
 	    case 'X':
+	    case 27:
 	    case 24:
 		endloop = 1;
 	    }
 	if (br > 0) {
-	    if ((fromaddr.spkt_family == ARPHRD_ETHER) ||
-	        (fromaddr.spkt_family == ARPHRD_FDDI)) {
-		if (fromaddr.spkt_protocol == htons(ETH_P_IP))
+	    linktype = getlinktype(fromaddr.sll_hatype, ifname, -1, NULL);
+
+	    if ((linktype == LINK_ETHERNET) || (linktype == LINK_FDDI) || (linktype == LINK_PLIP)) {
+		if (fromaddr.sll_protocol == htons(ETH_P_IP))
 		    is_ip = 1;
 		else
 		    is_ip = 0;
@@ -511,21 +553,20 @@ void hostmon(const struct OPTIONS *options, int facilitytime)
 		/*
 		 * Check source address entry
 		 */
-		
-		if (fromaddr.spkt_family == ARPHRD_ETHER) {
+
+		if ((linktype == LINK_ETHERNET) || (linktype == LINK_PLIP)) {
 		    memcpy(scratch_saddr, ((struct ethhdr *) buf)->h_source, ETH_ALEN);
 		    memcpy(scratch_daddr, ((struct ethhdr *) buf)->h_dest, ETH_ALEN);
 		    list = &elist;
-		} else if (fromaddr.spkt_family == ARPHRD_FDDI) {
+		} else if (linktype == LINK_FDDI) {
 		    memcpy(scratch_saddr, ((struct fddihdr *) buf)->saddr, FDDI_K_ALEN);
 		    memcpy(scratch_daddr, ((struct fddihdr *) buf)->daddr, FDDI_K_ALEN);
 		    list = &flist;
 		}
-		    
-		entry = in_ethtable(&table, fromaddr.spkt_family, scratch_saddr);
+		entry = in_ethtable(&table, linktype, scratch_saddr);
 
 		if ((entry == NULL) && (!nomem))
-		    entry = addethentry(&table, fromaddr.spkt_family, fromaddr.spkt_device, scratch_saddr, &nomem, list);
+		    entry = addethentry(&table, linktype, ifname, scratch_saddr, &nomem, list);
 
 		if (entry != NULL) {
 		    updateethent(entry, br, is_ip, 1);
@@ -538,9 +579,9 @@ void hostmon(const struct OPTIONS *options, int facilitytime)
 		 * Check destination address entry
 		 */
 
-		entry = in_ethtable(&table, fromaddr.spkt_family, scratch_daddr);
+		entry = in_ethtable(&table, linktype, scratch_daddr);
 		if ((entry == NULL) && (!nomem))
-		    entry = addethentry(&table, fromaddr.spkt_family, fromaddr.spkt_device, scratch_daddr, &nomem, list);
+		    entry = addethentry(&table, linktype, ifname, scratch_daddr, &nomem, list);
 
 		if (entry != NULL) {
 		    updateethent(entry, br, is_ip, 0);
@@ -551,33 +592,35 @@ void hostmon(const struct OPTIONS *options, int facilitytime)
 		}
 	    }
 	}
-        gettimeofday(&tv, NULL);
+	gettimeofday(&tv, NULL);
 	now = tv.tv_sec;
-	unow = tv.tv_usec;
-	
+	unow = tv.tv_sec * 1e+6 + tv.tv_usec;
+
 	if ((now - starttime) >= 5) {
 	    printelapsedtime(statbegin, now, LINES - 3, 15, table.borderwin);
-	    updateethrates(&table, starttime, now, idx);
+	    updateethrates(&table, options->actmode, starttime, now, idx);
 	    starttime = now;
 	}
 	if (((now - startlog) >= options->logspan) && (logging)) {
-	    writeethlog(table.head, now - statbegin, logfile);
+	    writeethlog(table.head, options->actmode, now - statbegin, logfile);
 	    startlog = now;
 	}
 	if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-	   ((options->updrate == 0) && (unow - updtime_usec >= HOSTMON_UPDATE_DELAY))) {
+	    ((options->updrate == 0) && (unow - updtime_usec >= HOSTMON_UPDATE_DELAY))) {
 	    update_panels();
 	    doupdate();
 	    updtime = now;
 	    updtime_usec = unow;
 	}
-	
+	check_rotate_flag(&logfile, logging);
+
 	if ((facilitytime != 0) && (((now - statbegin) / 60) >= facilitytime))
 	    endloop = 1;
     } while (!endloop);
 
     if (logging) {
-	writeethlog(table.head, time((time_t *) NULL) - statbegin, logfile);
+	signal(SIGUSR1, SIG_DFL);
+	writeethlog(table.head, options->actmode, time((time_t *) NULL) - statbegin, logfile);
 	writelog(logging, logfile, "******** LAN traffic monitor stopped");
 	fclose(logfile);
     }
