@@ -21,6 +21,7 @@ details.
 #include <panel.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -37,6 +38,7 @@ details.
 #include <linux/if_ether.h>
 #include "ifstats.h"
 #include "ifaces.h"
+#include "isdntab.h"
 #include "packet.h"
 #include "log.h"
 #include "dirs.h"
@@ -49,18 +51,40 @@ details.
 #include "timer.h"
 #include "options.h"
 #include "instances.h"
+#include "mode.h"
+#include "logvars.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
 
 /* from log.c, applicable only to this module */
 
-void writegstatlog(struct iftab *table, unsigned long nsecs, FILE * logfile);
-void writedstatlog(char *ifname, float activity, float pps,
+void writegstatlog(struct iftab *table, int unit, unsigned long nsecs, FILE * logfile);
+void writedstatlog(char *ifname, int unit, float activity, float pps,
 		   float peakactivity, float peakpps,
+		   float peakactivity_in, float peakpps_in,
+		   float peakactivity_out, float peakpps_out,
 		   struct iftotals *ts, unsigned long nsecs,
 		   FILE * logfile);
 
+/*
+ * USR1 log-rotation signal handlers
+ */
+
+void rotate_gstat_log()
+{
+    rotate_flag = 1;
+    strcpy(target_logname, GSTATLOG);
+    signal(SIGUSR1, rotate_gstat_log);
+}
+
+void rotate_dstat_log()
+{
+    rotate_flag = 1;
+    strcpy(target_logname, DSTATLOG);
+    signal(SIGUSR1, rotate_dstat_log);
+}
+
 char *ltrim(char *buf)
 {
     char *tmp = buf;
@@ -90,6 +114,16 @@ int ifinlist(struct iflist *list, char *ifname)
     return result;
 }
 
+/*
+ * Initialize the list of interfaces.  This linked list is used in the
+ * selection boxes as well as in the general interface statistics screen.
+ *
+ * This function parses the /proc/net/dev file and grabs the interface names
+ * from there.  The SIOGIFFLAGS ioctl() call is used to determine whether the
+ * interfaces are active.  Inactive interfaces are omitted from selection
+ * lists.
+ */
+
 void initiflist(struct iflist **list)
 {
     FILE *fd;
@@ -122,6 +156,19 @@ void initiflist(struct iflist **list)
 	    if (ifinlist(*list, ifname))	/* ignore entry if already in */
 		continue;	/* interface list */
 
+	    /*
+	     * Check if the interface is actually up running.  This prevents
+	     * inactive devices in /proc/net/dev from actually appearing in
+	     * interface lists used by IPTraf.
+	     */
+
+	    if (!iface_up(ifname))
+		continue;
+
+	    /*
+	     * At this point, the interface is now sure to be up and running.
+	     */
+
 	    itmp = malloc(sizeof(struct iflist));
 	    bzero(itmp, sizeof(struct iflist));
 	    strcpy(itmp->ifname, ifname);
@@ -169,9 +216,9 @@ void positionptr(struct iftab *table, struct iflist **ptmp, char *ifname)
 	(*ptmp)->prev_entry = plast;
 	(*ptmp)->next_entry = NULL;
 	strcpy((*ptmp)->ifname, ifname);
-	
+
 	if ((*ptmp)->index <= LINES - 4)
-	   table->lastvisible = *ptmp;
+	    table->lastvisible = *ptmp;
     }
 }
 
@@ -193,20 +240,25 @@ void destroyiflist(struct iflist *list)
     }
 }
 
-void updaterates(struct iftab *table, time_t starttime, time_t now,
+void updaterates(struct iftab *table, int unit, time_t starttime, time_t now,
 		 unsigned int idx)
 {
     struct iflist *ptmp = table->firstvisible;
 
     wattrset(table->statwin, HIGHATTR);
     do {
-	ptmp->rate = ((float) (ptmp->spanbr * 8 / 1000)) / ((float) (now - starttime));
+	wmove(table->statwin, ptmp->index - idx, 52 * COLS / 80);
+	if (unit == KBITS) {
+	    ptmp->rate = ((float) (ptmp->spanbr * 8 / 1000)) / ((float) (now - starttime));
+	    wprintw(table->statwin, "%8.2f kbits/sec", ptmp->rate);
+	} else {
+	    ptmp->rate = ((float) (ptmp->spanbr / 1024)) / ((float) (now - starttime));
+	    wprintw(table->statwin, "%8.2f kbytes/sec", ptmp->rate);
+	}
 
 	if (ptmp->rate > ptmp->peakrate)
 	    ptmp->peakrate = ptmp->rate;
 
-	wmove(table->statwin, ptmp->index - idx, 52);
-	wprintw(table->statwin, "%8.2f kbits/sec", ptmp->rate);
 	ptmp->spanbr = 0;
 	ptmp = ptmp->next_entry;
     } while (ptmp != table->lastvisible->next_entry);
@@ -225,13 +277,13 @@ void printifentry(struct iflist *ptmp, WINDOW * win, unsigned int idx)
     wmove(win, target_row, 1);
     wprintw(win, "%s", ptmp->ifname);
     wattrset(win, HIGHATTR);
-    wmove(win, target_row, 12);
+    wmove(win, target_row, 12 * COLS / 80);
     printlargenum(ptmp->total, win);
-    wmove(win, target_row, 22);
+    wmove(win, target_row, 22 * COLS / 80);
     printlargenum(ptmp->iptotal, win);
-    wmove(win, target_row, 32);
+    wmove(win, target_row, 32 * COLS / 80);
     printlargenum(ptmp->noniptotal, win);
-    wmove(win, target_row, 42);
+    wmove(win, target_row, 42 * COLS / 80);
     wprintw(win, "%8lu", ptmp->badtotal);
 }
 
@@ -259,15 +311,15 @@ void labelstats(WINDOW * win)
 {
     wmove(win, 0, 1);
     wprintw(win, " Iface ");
-    wmove(win, 0, 16);
+    wmove(win, 0, 16 * COLS / 80);
     wprintw(win, " Total ");
-    wmove(win, 0, 29);
+    wmove(win, 0, 29 * COLS / 80);
     wprintw(win, " IP ");
-    wmove(win, 0, 36);
+    wmove(win, 0, 36 * COLS / 80);
     wprintw(win, " NonIP ");
-    wmove(win, 0, 45);
+    wmove(win, 0, 45 * COLS / 80);
     wprintw(win, " BadIP ");
-    wmove(win, 0, 55);
+    wmove(win, 0, 55 * COLS / 80);
     wprintw(win, " Activity ");
 }
 
@@ -275,7 +327,7 @@ void initiftab(struct iftab *table)
 {
     initiflist(&(table->head));
 
-    table->borderwin = newwin(LINES - 2, 80, 1, 0);
+    table->borderwin = newwin(LINES - 2, COLS, 1, 0);
     table->borderpanel = new_panel(table->borderwin);
 
     move(LINES - 1, 1);
@@ -284,14 +336,14 @@ void initiftab(struct iftab *table)
     wattrset(table->borderwin, BOXATTR);
     box(table->borderwin, ACS_VLINE, ACS_HLINE);
     labelstats(table->borderwin);
-    table->statwin = newwin(LINES - 4, 78, 2, 1);
+    table->statwin = newwin(LINES - 4, COLS - 2, 2, 1);
     table->statpanel = new_panel(table->statwin);
     stdwinset(table->statwin);
     wtimeout(table->statwin, -1);
     wattrset(table->statwin, STDATTR);
     colorwin(table->statwin);
     wattrset(table->statwin, BOXATTR);
-    wmove(table->borderwin, LINES - 3, 32);
+    wmove(table->borderwin, LINES - 3, 32 * COLS / 80);
     wprintw(table->borderwin, " Total, IP, NonIP, and BadIP are packet counts ");
 }
 
@@ -301,6 +353,8 @@ void initiftab(struct iftab *table)
 
 void scrollgstatwin(struct iftab *table, int direction, unsigned int *idx)
 {
+    char buf[255];
+    sprintf(buf, "%%%dc", COLS - 2);
     wattrset(table->statwin, STDATTR);
     if (direction == SCROLLUP) {
 	if (table->lastvisible->next_entry != NULL) {
@@ -310,7 +364,7 @@ void scrollgstatwin(struct iftab *table, int direction, unsigned int *idx)
 	    (*idx)++;
 	    wmove(table->statwin, LINES - 5, 0);
 	    scrollok(table->statwin, 0);
-	    wprintw(table->statwin, "%78c", ' ');
+	    wprintw(table->statwin, buf, ' ');
 	    scrollok(table->statwin, 1);
 	    printifentry(table->lastvisible, table->statwin, *idx);
 	}
@@ -321,7 +375,7 @@ void scrollgstatwin(struct iftab *table, int direction, unsigned int *idx)
 	    table->lastvisible = table->lastvisible->prev_entry;
 	    (*idx)--;
 	    wmove(table->statwin, 0, 0);
-	    wprintw(table->statwin, "%78c", ' ');
+	    wprintw(table->statwin, buf, ' ');
 	    printifentry(table->firstvisible, table->statwin, *idx);
 	}
     }
@@ -354,12 +408,16 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
     int logging = options->logging;
     struct iftab table;
 
+    struct isdntab isdntable;
+
     char buf[8192];
+    char aligned_buf[120];
     char *packet;
     struct iphdr *ipacket;
     char *tpacket;
 
-    struct sockaddr_pkt fromaddr;
+    struct sockaddr_ll fromaddr;
+    unsigned short linktype;
 
     struct iflist *ptmp = NULL;
 
@@ -367,6 +425,7 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 
     int fd;
     FILE *logfile = NULL;
+    int isdn_fd = -1;
 
     int br;
     char ifname[10];
@@ -381,19 +440,25 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
     unsigned long starttime = 0;
     unsigned long statbegin = 0;
     unsigned long now = 0;
-    unsigned long unow = 0;
+    unsigned long long unow = 0;
     unsigned long startlog = 0;
     unsigned long updtime = 0;
-    unsigned long updtime_usec = 0;
+    unsigned long long updtime_usec = 0;
 
     if (!facility_active(GSTATIDFILE))
-        mark_facility(GSTATIDFILE, "general interface statistics");
+	mark_facility(GSTATIDFILE, "general interface statistics");
     else {
-        errbox("General interface stats already active in another process",
-               ANYKEY_MSG, &ch);
-        return;
+	errbox("General interface stats already active in another process",
+	       ANYKEY_MSG, &ch);
+	return;
+    }
+
+    open_socket(&fd);
+
+    if (fd < 0) {
+	unlink(GSTATIDFILE);
+	return;
     }
-        
     initiftab(&table);
 
     if (logging) {
@@ -402,24 +467,25 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 	if (logfile == NULL)
 	    logging = 0;
     }
+    if (logging)
+	signal(SIGUSR1, rotate_gstat_log);
+
+    rotate_flag = 0;
     writelog(logging, logfile, "******** General interface statistics started");
 
     if (table.head != NULL) {
-	open_socket(&fd, "");
-
-	if (fd < 0)
-	    return;
-
 	preparescreen(&table);
 
 	update_panels();
 	doupdate();
 
+	bzero(&isdntable, sizeof(struct isdntab));
+
 	gettimeofday(&tv, NULL);
 	starttime = startlog = statbegin = tv.tv_sec;
 
 	while (!endloop) {
-	    getpacket(fd, buf, &fromaddr, &ch, &br, 5, table.statwin);
+	    getpacket(fd, buf, &fromaddr, &ch, &br, ifname, table.statwin);
 
 	    if (ch != ERR) {
 		switch (ch) {
@@ -448,7 +514,6 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 		}
 	    }
 	    if (br > 0) {
-		strcpy(ifname, fromaddr.spkt_device);
 		positionptr(&table, &ptmp, ifname);
 
 		ptmp->total++;
@@ -456,10 +521,14 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 		ptmp->spanbr += br;
 		ptmp->br += br;
 
-		if (fromaddr.spkt_protocol == ntohs(ETH_P_IP)) {
+		if (fromaddr.sll_protocol == ntohs(ETH_P_IP)) {
 		    (ptmp->iptotal)++;
-		    adjustpacket(buf, fromaddr.spkt_family,
-				 &packet, &br);
+
+		    isdn_iface_check(&isdn_fd, ifname);
+		    linktype = getlinktype(fromaddr.sll_hatype, ifname,
+					   isdn_fd, &isdntable);
+
+		    adjustpacket(buf, linktype, &packet, aligned_buf, &br);
 
 		    if (packet == NULL)
 			continue;
@@ -482,29 +551,30 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 		}
 		printifentry(ptmp, table.statwin, idx);
 	    }
-	    
 	    gettimeofday(&tv, NULL);
 	    now = tv.tv_sec;
-	    unow = tv.tv_usec;
-	    
-	    if ((now - starttime) >= 5) {
-		updaterates(&table, starttime, now, idx);
+	    unow = tv.tv_sec * 1e+6 + tv.tv_usec;
+
+	    if ((now - starttime) >= 7) {
+		updaterates(&table, options->actmode, starttime, now, idx);
 		printelapsedtime(statbegin, now, LINES - 3, 1, table.borderwin);
 		starttime = now;
 	    }
 	    if (((now - startlog) >= options->logspan) && (logging)) {
-		writegstatlog(&table, time((time_t *) NULL) - statbegin, logfile);
+		writegstatlog(&table, options->actmode, time((time_t *) NULL) - statbegin, logfile);
 		startlog = now;
 	    }
 	    if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-	       ((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
+		((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
 		update_panels();
 		doupdate();
 		updtime = now;
 		updtime_usec = unow;
 	    }
+	    check_rotate_flag(&logfile, logging);
+
 	    if ((facilitytime != 0) && (((now - statbegin) / 60) >= facilitytime))
-	        endloop = 1;
+		endloop = 1;
 	}
 
 	close(fd);
@@ -517,180 +587,108 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
     doupdate();
 
     if (logging) {
-	writegstatlog(&table, time((time_t *) NULL) - statbegin, logfile);
+	signal(SIGUSR1, SIG_DFL);
+	writegstatlog(&table, options->actmode, time((time_t *) NULL) - statbegin, logfile);
 	writelog(logging, logfile, "******** General interface statistics stopped");
 	fclose(logfile);
     }
     destroyiflist(table.head);
+    destroy_isdn_table(&isdntable);
     unlink(GSTATIDFILE);
 }
 
-int initialize_brackets(char *ifname, struct iftotals *ttmp)
-{
-    struct ifreq ifr;
-    int fd;
-    int resp;
-    int istat;
-    unsigned int interval;
-    int i;
-    
-    strcpy(ifr.ifr_name, ifname);
-    
-    fd = socket(PF_INET, SOCK_PACKET, htons(ETH_P_ALL));
-    
-    if (fd < 0) {
-        errbox("Unable to open socket for MTU determination", ANYKEY_MSG, &resp);
-        return 1;
-    }
-    
-    istat = ioctl(fd, SIOCGIFMTU, &ifr);
-    
-    close(fd);
-    if (istat < 0) {
-        errbox("Unable to obtain interface MTU", ANYKEY_MSG, &resp);
-	return 1;
-    }
-            
-    ttmp->interval = interval = ifr.ifr_mtu / 16;    /* There are 16 packet size brackets */
-    
-    for (i = 0; i <= 15; i++) {
-        ttmp->brackets[i].floor = interval * i + 1;
-        ttmp->brackets[i].ceil = interval * (i + 1);
-    }
-    
-    ttmp->brackets[15].ceil = ifr.ifr_mtu;
-    return 0;
-}
 
 void printdetlabels(WINDOW * win, struct iftotals *totals)
 {
-    char *sizelabels = "Pkt Size (bytes)        Count";
-    int i;
-
+    /* TODO? : Adjust values for !80 column displays */
     wattrset(win, BOXATTR);
-    wmove(win, 2, 14);
-    wprintw(win, "Packets       Bytes");
-    wmove(win, 12, 2);
-    wprintw(win, sizelabels);
-    wmove(win, 12, 36);
-    wprintw(win, sizelabels);
+    mvwprintw(win, 2, 14, "  Total      Total    Incoming   Incoming    Outgoing   Outgoing");
+    mvwprintw(win, 3, 14, "Packets      Bytes     Packets      Bytes     Packets      Bytes");
     wattrset(win, STDATTR);
-    wmove(win, 3, 2);
-    wprintw(win, "Total:");
-    wmove(win, 4, 2);
-    wprintw(win, "IP:");
-    wmove(win, 5, 2);
-    wprintw(win, "TCP:");
-    wmove(win, 6, 2);
-    wprintw(win, "UDP:");
-    wmove(win, 7, 2);
-    wprintw(win, "ICMP:");
-    wmove(win, 8, 2);
-    wprintw(win, "Other IP:");
-    wmove(win, 9, 2);
-    wprintw(win, "Non-IP:");
-    wmove(win, 5, 40);
-    wprintw(win, "Activity:");
+    mvwprintw(win, 4, 2, "Total:");
+    mvwprintw(win, 5, 2, "IP:");
+    mvwprintw(win, 6, 2, "TCP:");
+    mvwprintw(win, 7, 2, "UDP:");
+    mvwprintw(win, 8, 2, "ICMP:");
+    mvwprintw(win, 9, 2, "Other IP:");
+    mvwprintw(win, 10, 2, "Non-IP:");
+    mvwprintw(win, 13, 2, "Total rates:");
+    mvwprintw(win, 16, 2, "Incoming rates:");
+    mvwprintw(win, 19, 2, "Outgoing rates:");
 
-    wmove(win, 8, 40);
-    wprintw(win, "IP Checksum Errors:");
-
-
-    for (i = 0; i <= 7; i++) {
-        wattrset(win, STDATTR);
-        wmove(win, i + 13, 2);
-        wprintw(win, "%4u to %4u:", totals->brackets[i].floor, totals->brackets[i].ceil);
-        wmove(win, i+ 13, 23);
-        wattrset(win, HIGHATTR);
-        wprintw(win, "%8lu", 0);
-    }
-
-    for (i = 8; i <= 15; i++) {
-        wattrset(win, STDATTR);
-        wmove(win, i + 5, 36);
-        wprintw(win, "%4u to %4u:", totals->brackets[i].floor, totals->brackets[i].ceil);
-        wmove(win, i + 5, 57);
-        wattrset(win, HIGHATTR);
-        wprintw(win, "%8lu", 0);
-    }
+    wmove(win, 16, 45);
+    wprintw(win, "IP checksum errors:");
 
     update_panels();
     doupdate();
 }
 
+void printstatrow(WINDOW * win, int row,
+		  unsigned long long total, unsigned long long btotal,
+	       unsigned long long total_in, unsigned long long btotal_in,
+	     unsigned long long total_out, unsigned long long btotal_out)
+{
+    wmove(win, row, 12);
+    printlargenum(total, win);
+    wmove(win, row, 23);
+    printlargenum(btotal, win);
+    wmove(win, row, 35);
+    printlargenum(total_in, win);
+    wmove(win, row, 46);
+    printlargenum(btotal_in, win);
+    wmove(win, row, 58);
+    printlargenum(total_out, win);
+    wmove(win, row, 69);
+    printlargenum(btotal_out, win);
+}
+
 void printdetails(struct iftotals *totals, WINDOW * win)
 {
     wattrset(win, HIGHATTR);
 
     /* Print totals on the IP protocols */
 
-    wmove(win, 3, 12);
-    printlargenum(totals->total, win);
-    wmove(win, 3, 24);
-    printlargenum(totals->bytestotal, win);
-    wmove(win, 4, 12);
-    printlargenum(totals->iptotal, win);
-    wmove(win, 4, 24);
-    printlargenum(totals->ipbtotal, win);
+    printstatrow(win, 4, totals->total, totals->bytestotal,
+		 totals->total_in, totals->bytestotal_in,
+		 totals->total_out, totals->bytestotal_out);
 
-    wmove(win, 5, 12);
-    printlargenum(totals->tcptotal, win);
-    wmove(win, 5, 24);
-    printlargenum(totals->tcpbtotal, win);
+    printstatrow(win, 5, totals->iptotal, totals->ipbtotal,
+		 totals->iptotal_in, totals->ipbtotal_in,
+		 totals->iptotal_out, totals->ipbtotal_out);
 
-    wmove(win, 6, 12);
-    printlargenum(totals->udptotal, win);
-    wmove(win, 6, 24);
-    printlargenum(totals->udpbtotal, win);
+    printstatrow(win, 6, totals->tcptotal, totals->tcpbtotal,
+		 totals->tcptotal_in, totals->tcpbtotal_in,
+		 totals->tcptotal_out, totals->tcpbtotal_out);
 
-    wmove(win, 7, 12);
-    printlargenum(totals->icmptotal, win);
-    wmove(win, 7, 24);
-    printlargenum(totals->icmpbtotal, win);
+    printstatrow(win, 7, totals->udptotal, totals->udpbtotal,
+		 totals->udptotal_in, totals->udpbtotal_in,
+		 totals->udptotal_out, totals->udpbtotal_out);
 
-    wmove(win, 8, 12);
-    printlargenum(totals->othtotal, win);
-    wmove(win, 8, 24);
-    printlargenum(totals->othbtotal, win);
+    printstatrow(win, 8, totals->icmptotal, totals->icmpbtotal,
+		 totals->icmptotal_in, totals->icmpbtotal_in,
+		 totals->icmptotal_out, totals->icmpbtotal_out);
+
+    printstatrow(win, 9, totals->othtotal, totals->othbtotal,
+		 totals->othtotal_in, totals->othbtotal_in,
+
+		 totals->othtotal_out, totals->othbtotal_out);
 
     /* Print non-IP totals */
 
-    wmove(win, 9, 12);
-    printlargenum(totals->noniptotal, win);
-    wmove(win, 9, 24);
-    printlargenum(totals->nonipbtotal, win);
-
+    printstatrow(win, 10, totals->noniptotal, totals->nonipbtotal,
+		 totals->noniptotal_in, totals->nonipbtotal_in,
+		 totals->noniptotal_out, totals->nonipbtotal_out);
     /* Bad packet count */
 
-    wmove(win, 8, 60);
+    wmove(win, 16, 67);
     wprintw(win, "%8lu", totals->badtotal);
 }
 
-void update_size_distrib(unsigned int length, struct iftotals *totals, WINDOW *win)
-{
-    unsigned int i;
-    
-    i = (length - 1) / totals->interval;   /* minus 1 to keep interval
-    					      boundary lengths within the
-    					      proper brackets */
-    
-    if (i > 15)				   /* This is for extras for MTU's not */
-        i = 15;				   /* divisible by 16 */
-        
-    totals->brackets[i].count++;
-    
-    if (i < 8)
-        wmove(win, i + 13, 23);
-    else
-        wmove(win, i + 5, 57);
-        
-    wprintw(win, "%8lu", totals->brackets[i].count);
-}
 
 /*
  * The detailed interface statistics function
  */
- 
+
 void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 {
     int logging = options->logging;
@@ -699,60 +697,93 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     PANEL *statpanel;
 
     char buf[8192];
+    char aligned_buf[120];
     char *packet;
     struct iphdr *ipacket = NULL;
     char *tpacket;
 
-    struct sockaddr_pkt fromaddr;
+    char ifname[8];
+    struct sockaddr_ll fromaddr;
+    unsigned short linktype;
 
     int fd;
     FILE *logfile = NULL;
+    int isdn_fd = -1;
     int br;
     int framelen = 0;
 
     int iphlen;
     int ipck, ipcsum;
-    unsigned int iplen;
+    unsigned int iplen = 0;
 
     struct iftotals totals;
 
     int ch;
 
+    struct isdntab isdntable;
+
     struct timeval tv;
     unsigned long updtime = 0;
-    unsigned long updtime_usec = 0;
+    unsigned long long updtime_usec = 0;
     int endloop = 0;
     unsigned long starttime, now;
     unsigned long statbegin, startlog;
-    unsigned long unow;
+    unsigned long long unow;
+
     float spanbr = 0;
     float spanpkt = 0;
+    float spanbr_in = 0;
+    float spanbr_out = 0;
+    float spanpkt_in = 0;
+    float spanpkt_out = 0;
+
     float activity = 0;
+    float activity_in = 0;
+    float activity_out = 0;
     float peakactivity = 0;
+    float peakactivity_in = 0;
+    float peakactivity_out = 0;
+
     float pps = 0;
     float peakpps = 0;
+    float pps_in = 0;
+    float pps_out = 0;
+    float peakpps_in = 0;
+    float peakpps_out = 0;
+
+    char unitstring[7];
 
     /*
      * Mark this facility
      */
-     
+
     if (!facility_active(DSTATIDFILE))
-        mark_facility(DSTATIDFILE, "detailed interface statistics");
+	mark_facility(DSTATIDFILE, "detailed interface statistics");
     else {
-        errbox("Detailed interface stats already active in another process",
-               ANYKEY_MSG, &ch);
-        return;
+	errbox("Detailed interface stats already active in another process",
+	       ANYKEY_MSG, &ch);
+	return;
+    }
+
+    open_socket(&fd);
+
+    if (fd < 0) {
+	unlink(DSTATIDFILE);
+	return;
     }
-    
     if (!iface_supported(iface)) {
-        err_iface_unsupported();
-        unlink(DSTATIDFILE);
-        return;
+	err_iface_unsupported();
+	unlink(DSTATIDFILE);
+	return;
+    }
+    if (!iface_up(iface)) {
+	err_iface_down();
+	unlink(DSTATIDFILE);
+	return;
     }
-        
     move(LINES - 1, 1);
     stdexitkeyhelp();
-    statwin = newwin(23, 80, 1, 0);
+    statwin = newwin(LINES - 2, COLS, 1, 0);
     statpanel = new_panel(statwin);
     stdwinset(statwin);
     wtimeout(statwin, -1);
@@ -766,47 +797,41 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     doupdate();
 
     bzero(&totals, sizeof(struct iftotals));
-    
+
     if (logging) {
 	opentlog(&logfile, DSTATLOG);
 
 	if (logfile == NULL)
 	    logging = 0;
     }
+    if (logging)
+	signal(SIGUSR1, rotate_dstat_log);
+
+    rotate_flag = 0;
     writelog(logging, logfile, "******** Detailed interface statistics started");
-    open_socket(&fd, iface);
 
-    if (fd < 0)
-	return;
-
-    if (initialize_brackets(iface, &totals) != 0) {
-        close(fd);
-        del_panel(statpanel);
-        delwin(statwin);
-        update_panels();
-        doupdate();
-        unlink(DSTATIDFILE);
-        return;
-    }
-   
     printdetlabels(statwin, &totals);
     printdetails(&totals, statwin);
     update_panels();
     doupdate();
 
     spanbr = 0;
-    
+
+    bzero(&isdntable, sizeof(struct isdntab));
+
     gettimeofday(&tv, NULL);
     starttime = startlog = statbegin = tv.tv_sec;
-    
+
     leaveok(statwin, TRUE);
 
+    dispmode(options->actmode, unitstring);
+
     /*
      * Data-gathering loop
      */
-     
+
     while (!endloop) {
-	getpacket(fd, buf, &fromaddr, &ch, &br, 5, statwin);
+	getpacket(fd, buf, &fromaddr, &ch, &br, ifname, statwin);
 
 	if (ch != ERR) {
 	    switch (ch) {
@@ -821,20 +846,34 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 	    }
 	}
 	if (br > 0) {
-	    if (strcmp(iface, fromaddr.spkt_device) != 0)
+	    if (strcmp(iface, ifname) != 0)
 		continue;
 
 	    framelen = br;
 	    totals.total++;
 	    totals.bytestotal += br;
 
+	    if (fromaddr.sll_pkttype == PACKET_OUTGOING) {
+		totals.total_out++;
+		totals.bytestotal_out += br;
+		spanbr_out += br;
+		spanpkt_out++;
+	    } else {
+		totals.total_in++;
+		totals.bytestotal_in += br;
+		spanbr_in += br;
+		spanpkt_in++;
+	    }
+
 	    spanbr += br;
 	    spanpkt++;
 
-	    if (fromaddr.spkt_protocol == ntohs(ETH_P_IP)) {
-		totals.iptotal++;
-		adjustpacket(buf, fromaddr.spkt_family,
-			     &packet, &br);
+	    if (fromaddr.sll_protocol == ntohs(ETH_P_IP)) {
+		isdn_iface_check(&isdn_fd, ifname);
+		linktype = getlinktype(fromaddr.sll_hatype, ifname,
+				       isdn_fd, &isdntable);
+
+		adjustpacket(buf, linktype, &packet, aligned_buf, &br);
 
 		if (packet == NULL)
 		    continue;
@@ -851,72 +890,150 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 		    continue;
 		}
 		iplen = ntohs(ipacket->tot_len);
+
+		totals.iptotal++;
 		totals.ipbtotal += iplen;
 
+		if (fromaddr.sll_pkttype == PACKET_OUTGOING) {
+		    totals.iptotal_out++;
+		    totals.ipbtotal_out += iplen;
+		} else {
+		    totals.iptotal_in++;
+		    totals.ipbtotal_in += iplen;
+		}
+
 		switch (ipacket->protocol) {
 		case IPPROTO_TCP:
 		    totals.tcptotal++;
 		    totals.tcpbtotal += iplen;
+
+		    if (fromaddr.sll_pkttype == PACKET_OUTGOING) {
+			totals.tcptotal_out++;
+			totals.tcpbtotal_out += iplen;
+		    } else {
+			totals.tcptotal_in++;
+			totals.tcpbtotal_in += iplen;
+		    }
 		    break;
 		case IPPROTO_UDP:
 		    totals.udptotal++;
 		    totals.udpbtotal += iplen;
+
+		    if (fromaddr.sll_pkttype == PACKET_OUTGOING) {
+			totals.udptotal_out++;
+			totals.udpbtotal_out += iplen;
+		    } else {
+			totals.udptotal_in++;
+			totals.udpbtotal_in += iplen;
+		    }
 		    break;
 		case IPPROTO_ICMP:
 		    totals.icmptotal++;
 		    totals.icmpbtotal += iplen;
+
+		    if (fromaddr.sll_pkttype == PACKET_OUTGOING) {
+			totals.icmptotal_out++;
+			totals.icmpbtotal_out += iplen;
+		    } else {
+			totals.icmptotal_in++;
+			totals.icmpbtotal_in += iplen;
+		    }
 		    break;
 		default:
 		    totals.othtotal++;
 		    totals.othbtotal += iplen;
+
+		    if (fromaddr.sll_pkttype == PACKET_OUTGOING) {
+			totals.othtotal_out++;
+			totals.othbtotal_out += iplen;
+		    } else {
+			totals.othtotal_in++;
+			totals.othbtotal_in += iplen;
+		    }
 		    break;
 		}
 	    } else {
 		totals.noniptotal++;
 		totals.nonipbtotal += br;
+
+		if (fromaddr.sll_pkttype == PACKET_OUTGOING) {
+		    totals.noniptotal_out++;
+		    totals.nonipbtotal_out += br;
+		} else {
+		    totals.noniptotal_in++;
+		    totals.nonipbtotal_in += br;
+		}
 	    }
-            update_size_distrib(framelen, &totals, statwin);
 	}
-	
 	gettimeofday(&tv, NULL);
 	now = tv.tv_sec;
-	unow = tv.tv_usec;
-	
-	if (now - starttime >= 5) {
+	unow = tv.tv_sec * 1e+6 + tv.tv_usec;
+
+	if (now - starttime >= 7) {
 	    wattrset(statwin, BOXATTR);
-	    printelapsedtime(statbegin, now, 22, 1, statwin);
-	    activity = (float) (spanbr * 8 / 1000) / (float) (now - starttime);
+	    printelapsedtime(statbegin, now, LINES - 3, 1, statwin);
+	    if (options->actmode == KBITS) {
+		activity = (float) (spanbr * 8 / 1000) / (float) (now - starttime);
+		activity_in = (float) (spanbr_in * 8 / 1000) / (float) (now - starttime);
+		activity_out = (float) (spanbr_out * 8 / 1000) / (float) (now - starttime);
+	    } else {
+		activity = (float) (spanbr / 1024) / (float) (now - starttime);
+		activity_in = (float) (spanbr_in / 1024) / (float) (now - starttime);
+		activity_out = (float) (spanbr_out / 1024) / (float) (now - starttime);
+	    }
+
 	    pps = (float) (spanpkt) / (float) (now - starttime);
-	    spanbr = 0;
-	    spanpkt = 0;
+	    pps_in = (float) (spanpkt_in) / (float) (now - starttime);
+	    pps_out = (float) (spanpkt_out) / (float) (now - starttime);
+
+	    spanbr = spanbr_in = spanbr_out = 0;
+	    spanpkt = spanpkt_in = spanpkt_out = 0;
 	    starttime = now;
 
 	    wattrset(statwin, HIGHATTR);
-	    wmove(statwin, 5, 50);
-	    wprintw(statwin, "%8.1f kbits/sec", activity);
-	    wmove(statwin, 6, 50);
-	    wprintw(statwin, "%8.1f packets/sec", pps);
+	    mvwprintw(statwin, 13, 19, "%8.1f %s/sec", activity, unitstring);
+	    mvwprintw(statwin, 14, 19, "%8.1f packets/sec", pps);
+	    mvwprintw(statwin, 16, 19, "%8.1f %s/sec", activity_in, unitstring);
+	    mvwprintw(statwin, 17, 19, "%8.1f packets/sec", pps_in);
+	    mvwprintw(statwin, 19, 19, "%8.1f %s/sec", activity_out, unitstring);
+	    mvwprintw(statwin, 20, 19, "%8.1f packets/sec", pps_out);
 
 	    if (activity > peakactivity)
 		peakactivity = activity;
 
+	    if (activity_in > peakactivity_in)
+		peakactivity_in = activity_in;
+
+	    if (activity_out > peakactivity_out)
+		peakactivity_out = activity_out;
+
 	    if (pps > peakpps)
 		peakpps = pps;
+
+	    if (pps_in > peakpps_in)
+		peakpps_in = pps_in;
+
+	    if (pps_out > peakpps_out)
+		peakpps_out = pps_out;
 	}
 	if (((now - startlog) >= options->logspan) && (logging)) {
-	    writedstatlog(iface, activity, pps, peakactivity, peakpps,
-		    &totals, time((time_t *) NULL) - statbegin, logfile);
+	    writedstatlog(iface, options->actmode, activity, pps,
+			  peakactivity, peakpps,
+			  peakactivity_in, peakpps_in,
+			  peakactivity_out, peakpps_out, &totals,
+			  time((time_t *) NULL) - statbegin, logfile);
 	    startlog = now;
 	}
 	if (((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY)) ||
-	   ((options->updrate != 0) && (now - updtime >= options->updrate))) {
-  	    printdetails(&totals, statwin);
+	    ((options->updrate != 0) && (now - updtime >= options->updrate))) {
+	    printdetails(&totals, statwin);
 	    update_panels();
 	    doupdate();
 	    updtime_usec = unow;
 	    updtime = now;
 	}
-	
+	check_rotate_flag(&logfile, logging);
+
 	if ((facilitytime != 0) && (((now - statbegin) / 60) >= facilitytime))
 	    endloop = 1;
     }
@@ -924,14 +1041,19 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     close(fd);
 
     if (logging) {
-	writedstatlog(iface, activity, pps, peakactivity, peakpps,
-		    &totals, time((time_t *) NULL) - statbegin, logfile);
+	signal(SIGUSR1, SIG_DFL);
+	writedstatlog(iface, options->actmode, activity, pps,
+		      peakactivity, peakpps,
+		      peakactivity_in, peakpps_in,
+		      peakactivity_out, peakpps_out, &totals,
+		      time((time_t *) NULL) - statbegin, logfile);
 	writelog(logging, logfile, "******** Detailed interface statistics stopped");
 	fclose(logfile);
     }
     del_panel(statpanel);
     delwin(statwin);
     unlink(DSTATIDFILE);
+    destroy_isdn_table(&isdntable);
     update_panels();
     doupdate();
 }
@@ -953,12 +1075,13 @@ void selectiface(char *ifname, int *aborted)
     initiflist(&list);
 
     listkeyhelp();
-    bw = newwin(14, 24, 6, 19);
+    bw = newwin(14, 24, (LINES - 14) / 2, (COLS - 24) / 2 - 9);
+    /* I'm guessing (COLS-24)/2-9*COLS/80) is overkill */
     bp = new_panel(bw);
     wattrset(bw, BOXATTR);
     box(bw, ACS_VLINE, ACS_HLINE);
 
-    win = newwin(12, 22, 7, 20);
+    win = newwin(12, 22, (LINES - 14) / 2 + 1, (COLS - 24) / 2 - 8);
     panel = new_panel(win);
     stdwinset(win);
     wattrset(win, BOXATTR);
