@@ -1,12 +1,13 @@
 /***
 
 IPTraf
-An IP-oriented Network Statistics Utility
+An IP Network Statistics Utility
    		
-Written by Gerard Paul Java
-Copyright (c) Gerard Paul Java 1997, 1998
+Written by Gerard Paul Java <riker@mozcom.com>
+Copyright (c) Gerard Paul Java 1997-1999
    	
-Version 1.3.0
+Version 2.0.0
+Main Module
 
 This software is open-source; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -52,29 +53,40 @@ extern void masq_warning();
 int check_ip_masquerade(void)
 {
     int fd;
-    
+
     fd = open("/proc/net/ip_masquerade", O_RDONLY);
-    
+
     if (fd >= 0) {
-        close(fd);
-        return 1;
+	close(fd);
+	return 1;
     }
-        
     return 0;
 }
 
+void init_break_menu(struct MENU *break_menu)
+{
+    initmenu(break_menu, 5, 20, (LINES - 5) / 2, COLS / 2);
+    additem(break_menu, " By packet ^s^ize", "Displays packet counts by packet size range");
+    additem(break_menu, " By TCP/UDP ^p^ort", "Displays packet and byte counts by service port");
+    additem(break_menu, " E^x^it menu", "Return to main menu");
+}
+
 /*
  * Get the ball rolling: The program interface routine.
  */
 
 void program_interface(struct OPTIONS *options,
-		       int opt, char *optarg, int facilitytime, 
-		       int is_first_instance)
+		       int opt, int dontwarn, char *optarg,
+		       int facilitytime, int is_first_instance)
 {
     struct MENU menu;
+    struct MENU break_menu;
+
     int endloop = 0;
     int row = 1;
+    int break_row = 1;
     int aborted;
+    int break_aborted;
     int faborted;
     int filtered = 0;
     struct filterlist fl;
@@ -85,25 +97,25 @@ void program_interface(struct OPTIONS *options,
     draw_desktop();
     attrset(STDATTR);
     move(0, 1);
-    printw("IPTraf %s", VERSION);
+    printw("IPTraf");
 
     loadsavedtcpfilter(&fl, &filtered);
     loadothfilter(&ofilter);
     indicate("");
     loadaddports(&ports);
 
-    if (check_ip_masquerade())
-        masq_warning();
-        
+    if ((!dontwarn) && (check_ip_masquerade()))
+	masq_warning();
+
     if (opt == 0) {
 	about();
 
-	initmenu(&menu, 14, 35, 5, 20);
+	initmenu(&menu, 14, 35, (LINES - 14) / 2, (COLS - 35) / 2);
 
 	additem(&menu, " IP traffic ^m^onitor", "Displays current IP traffic information");
 	additem(&menu, " General interface ^s^tatistics", "Displays some statistics for attached interfaces");
 	additem(&menu, " ^D^etailed interface statistics", "Displays more statistics for a selected interface");
-	additem(&menu, " TCP/UDP ser^v^ice monitor", "Displays statistics on TCP/UDP traffic");
+	additem(&menu, " Statistical ^b^reakdowns", "Facilities for traffic counts by port, packet size, or address");
 	additem(&menu, " ^L^AN station monitor", "Displays statistics on detected LAN stations");
 	additem(&menu, NULL, NULL);
 	additem(&menu, " ^T^CP display filters", "Manages TCP display filters");
@@ -133,9 +145,26 @@ void program_interface(struct OPTIONS *options,
 		    detstats(ifname, options, 0);
 		break;
 	    case 4:
-		selectiface(ifname, &aborted);
-		if (!aborted)
-		    servmon(ifname, ports, options, 0);
+		break_row = 1;
+		init_break_menu(&break_menu);
+		showmenu(&break_menu);
+		operatemenu(&break_menu, &break_row, &break_aborted);
+
+		switch (break_row) {
+		case 1:
+		    selectiface(ifname, &aborted);
+		    if (!aborted)
+			packet_size_breakdown(options, ifname, 0);
+		    break;
+		case 2:
+		    selectiface(ifname, &aborted);
+		    if (!aborted)
+			servmon(ifname, ports, options, 0);
+		    break;
+		case 4:
+		    break;
+		}
+		destroymenu(&break_menu);
 		break;
 	    case 5:
 		hostmon(options, 0);
@@ -162,7 +191,7 @@ void program_interface(struct OPTIONS *options,
 	switch (opt) {
 	case 'i':
 	    ipmon(options,
-		  filtered, &fl, &ofilter, options->timeout, facilitytime);
+		filtered, &fl, &ofilter, options->timeout, facilitytime);
 	    break;
 	case 'g':
 	    ifstats(options, facilitytime);
@@ -173,6 +202,9 @@ void program_interface(struct OPTIONS *options,
 	case 's':
 	    servmon(optarg, ports, options, facilitytime);
 	    break;
+	case 'z':
+	    packet_size_breakdown(options, optarg, facilitytime);
+	    break;
 	case 'l':
 	    hostmon(options, facilitytime);
 	    break;
@@ -196,10 +228,12 @@ void removetags()
     unlink(TCPFLTIDFILE);
     unlink(UDPFLTIDFILE);
     unlink(OTHFLTIDFILE);
+    unlink(PKTSIZEIDFILE);
 }
 
 /*
- * Handler for the TERM signal.  There's nothing we can do for the KILL.
+ * Handlers for the TERM signal and HUP signals.  There's nothing we can do
+ * for the KILL.
  */
 
 void termhandler()
@@ -213,6 +247,16 @@ void termhandler()
     exit(0);
 }
 
+/*
+ * Handler for the hangup signal.  Simply removes all tags.
+ */
+
+void huphandler()
+{
+    removetags();
+    exit(0);
+}
+
 /* 
  * Handler for the SIGSEGV, Segmentation Fault.  Tries to clear the screen
  * and issue a better message than "Segmentation fault".
@@ -236,16 +280,15 @@ void segvhandler()
 void commandhelp()
 {
     printf("\nSyntax:\n");
-    printf("    iptraf [ -f ][ { -i | -g | -d iface | -s iface | -l } [-t timeout ]]\n\n");
-    printf("Issuing the iptraf command without command-line options brings the program\n");
-    printf("up in interactive mode, with the various facilities accessed through a menu.\n\n");
+    printf("    iptraf [ -f ][ -q ][ { -i | -g | -d iface | -s iface | -z iface | -l }\n");
+    printf("           [-t timeout ]]\n\n");
+    printf("Issue the iptraf command with no parameters for menu-driven operation.\n");
     printf("These options can also be supplied to the command:\n\n");
-    printf("	-i		- immediately start the IP traffic monitor\n");
-    printf("	-g		- immediately start the general interface statistics\n");
-    printf("	-d iface	- allows you to immediately start the detailed\n");
-    printf("			  on the indicated interface (iface)\n");
-    printf("	-s iface	- allows you to immediately monitor TCP and UDP\n");
-    printf("			  traffic on the specified interface (iface)\n");
+    printf("	-i		- start the IP traffic monitor\n");
+    printf("	-g		- start the general interface statistics\n");
+    printf("	-d iface	- start the detailed statistics facility on an interface\n");
+    printf("	-s iface	- start the TCP and UDP monitor on an interface\n");
+    printf("	-z iface	- shows the packet size counts on an interface\n");
     printf("	-l		- start the LAN station monitor\n");
     printf("	-t timeout	- when used with one of the above parameters, tells\n");
     printf("			  the facility to run only for the specified number of\n");
@@ -253,8 +296,9 @@ void commandhelp()
     printf("	-f		- Remove all identification tag files.  This will\n");
     printf("			  allow multiple instances of facilities in different\n");
     printf("			  IPTraf processes.  Use with great caution.  May be\n");
-    printf("			  used after an abnormal termination.\n\n");
-    printf("IPTraf %s Copyright (c) Gerard Paul Java 1997, 1998\n", VERSION);
+    printf("			  used after an abnormal termination.\n");
+    printf("	-q		- suppress IP masquerading warning on startup\n\n");
+    printf("IPTraf %s Copyright (c) Gerard Paul Java 1997-1999\n", VERSION);
 }
 
 int first_instance()
@@ -277,8 +321,8 @@ void mark_first_instance()
 
     fd = open(IPTIDFILE, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR);
     if (fd < 0) {
-	fprintf(stderr, "\nWarning: unable to tag this process\n");
-	fprintf(stderr, "Press Enter to continue\n");
+	fprintf(stderr, "\nWarning: unable to tag this process\r\n");
+	fprintf(stderr, "Press Enter to continue\r\n");
 	getchar();
 	return;
     }
@@ -301,11 +345,12 @@ int main(int argc, char **argv)
     int is_first_instance;
     char keyparm[12];
     int facilitytime = 0;
+    int dontwarn = 0;
 
 #ifndef ALLOWUSERS
     if (getuid() != 0) {
 	fprintf(stderr, "\nIPTraf Version %s\n", VERSION);
-	fprintf(stderr, "Copyright (c) Gerard Paul Java 1997, 1998\n\n");
+	fprintf(stderr, "Copyright (c) Gerard Paul Java 1997-1999\n\n");
 	fprintf(stderr, "This program can be run only by the system administrator\n\n");
 	exit(1);
     }
@@ -318,19 +363,21 @@ int main(int argc, char **argv)
     if (argc > 1) {
 	do {
 	    opterr = 0;
-	    opt = getopt(argc, argv, "igd:s:lhft:");
+	    opt = getopt(argc, argv, "igd:s:z:lhfqt:");
 
 	    if (opt == 'h') {
 		commandhelp();
 		exit(0);
 	    } else if (opt == 'f') {
 		removetags();
-            } else if (opt == 't') {
-                facilitytime = atoi(optarg);
-                if (facilitytime == 0) {
-                    fprintf(stderr, "\nInvalid time value\n\n");
-                    exit(1);
-                }
+	    } else if (opt == 't') {
+		facilitytime = atoi(optarg);
+		if (facilitytime == 0) {
+		    fprintf(stderr, "\nInvalid time value\n\n");
+		    exit(1);
+		}
+	    } else if (opt == 'q') {
+		dontwarn = 1;
 	    } else if (opt == '?') {
 		fprintf(stderr, "\nInvalid option or missing parameter\n\n");
 		exit(1);
@@ -343,13 +390,7 @@ int main(int argc, char **argv)
 	    }
 	} while ((opt != '?') && (opt != -1));
     }
-
     is_first_instance = first_instance();
-            
-    signal(SIGTERM, (void *) termhandler);
-    signal(SIGSEGV, (void *) segvhandler);
-    signal(SIGTSTP, SIG_IGN);
-    signal(SIGINT, SIG_IGN);
 
     if (getenv("TERM") == NULL) {
 	fprintf(stderr, "Your TERM variable is not set.\n");
@@ -358,16 +399,23 @@ int main(int argc, char **argv)
     }
     loadoptions(&options);
 
-    mark_first_instance();
-
     initscr();
 
+    mark_first_instance();
+
     if ((LINES < 24) || (COLS < 80)) {
 	endwin();
 	fprintf(stderr, "\nThis program requires a screen size of at least 80 columns by 24 lines\n");
 	fprintf(stderr, "Please resize your window\n\n");
 	exit(1);
     }
+    signal(SIGTERM, (void *) termhandler);
+    signal(SIGHUP, (void *) huphandler);
+    signal(SIGSEGV, (void *) segvhandler);
+    signal(SIGTSTP, SIG_IGN);
+    signal(SIGINT, SIG_IGN);
+    signal(SIGUSR1, SIG_IGN);
+
     start_color();
     standardcolors(options.color);
     noecho();
@@ -379,18 +427,18 @@ int main(int argc, char **argv)
 #endif
 
     if (is_first_instance)
-        srpromisc(options.promisc);
+	srpromisc(options.promisc);
 
-    program_interface(&options, command,
-                      keyparm, facilitytime, is_first_instance);
+    program_interface(&options, command, dontwarn,
+		      keyparm, facilitytime, is_first_instance);
 
     if (is_first_instance)
-        srpromisc(0);
+	srpromisc(0);
 
     endwin();
 
     if (is_first_instance)
-        unlink(IPTIDFILE);
+	unlink(IPTIDFILE);
 
     return (0);
 }
