@@ -19,8 +19,10 @@ details.
 #include <sys/time.h>
 #include <sys/wait.h>
 #include <errno.h>
+#include <signal.h>
 #include "tcptable.h"
 #include "othptab.h"
+#include "isdntab.h"
 #include "packet.h"
 #include "ifaces.h"
 #include "ipcsum.h"
@@ -38,34 +40,56 @@ details.
 #include "ipfrag.h"
 #include "options.h"
 #include "instances.h"
+#include "logvars.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
 
+void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
+		 unsigned int pktlen, char *message);
+void write_tcp_unclosed(int logging, FILE * fd, struct tcptable *table);
+
+
+void rotate_ipmon_log()
+{
+    rotate_flag = 1;
+    strcpy(target_logname, IPMONLOG);
+    signal(SIGUSR1, rotate_ipmon_log);
+}
+
 /* Hot key indicators for the bottom line */
 
 void ipmonhelp()
 {
     move(LINES - 1, 1);
     printkeyhelp("Up/Dn/PgUp/PgDn", "-scrl actv win  ", stdscr);
+    move(LINES-1, 50);
     printkeyhelp("W", "-chg actv win  ", stdscr);
-    printkeyhelp("M", "-more TCP info  ", stdscr);
     stdexitkeyhelp();
 };
 
+void uniq_help(int what)
+{
+    move(LINES - 1, 32);
+    if (!what)
+        printkeyhelp("M", "-more TCP info  ", stdscr);
+    else
+        printkeyhelp("Lft/Rt", "-vert scrl ", stdscr);
+}  
+   
 /* Mark general packet count indicators */
 
 void prepare_ip_statwin(WINDOW * win)
 {
     wmove(win, 0, 1);
     wprintw(win, "IP:");
-    wmove(win, 0, 16);
+    wmove(win, 0, 16 * COLS / 80);
     wprintw(win, "TCP:");
-    wmove(win, 0, 31);
+    wmove(win, 0, 31 * COLS / 80);
     wprintw(win, "UDP:");
-    wmove(win, 0, 46);
+    wmove(win, 0, 46 * COLS / 80);
     wprintw(win, "ICMP:");
-    wmove(win, 0, 62);
+    wmove(win, 0, 62 * COLS / 80);
     wprintw(win, "Non-IP:");
 }
 
@@ -86,26 +110,28 @@ void markactive(int curwin, WINDOW * tw, WINDOW * ow)
     getmaxyx(win1, y1, x1);
     getmaxyx(win2, y2, x2);
 
-    wmove(win1, --y1, 68);
+    wmove(win1, --y1, COLS - 10);
+    wattrset(win1, ACTIVEATTR);
     wprintw(win1, " Active ");
-    wmove(win2, --y2, 68);
+    wattrset(win1, BOXATTR);
+    wmove(win2, --y2, COLS - 10);
     whline(win2, ACS_HLINE, 8);
 }
 
 void show_ip_stats(WINDOW * win, unsigned long long iptotal,
-                   unsigned long long tcptotal,
-		   unsigned long long udptotal, unsigned long long icmptotal,
+		   unsigned long long tcptotal,
+	       unsigned long long udptotal, unsigned long long icmptotal,
 		   unsigned long long noniptotal)
 {
-    wmove(win, 0, 6);
+    wmove(win, 0, 6 * COLS / 80);
     printlargenum(iptotal, win);
-    wmove(win, 0, 21);
+    wmove(win, 0, 21 * COLS / 80);
     printlargenum(tcptotal, win);
-    wmove(win, 0, 36);
+    wmove(win, 0, 36 * COLS / 80);
     printlargenum(udptotal, win);
-    wmove(win, 0, 52);
+    wmove(win, 0, 52 * COLS / 80);
     printlargenum(icmptotal, win);
-    wmove(win, 0, 70);
+    wmove(win, 0, 70 * COLS / 80);
     printlargenum(noniptotal, win);
 }
 
@@ -116,6 +142,8 @@ void show_ip_stats(WINDOW * win, unsigned long long iptotal,
 
 void scrollupperwin(struct tcptable *table, int direction, int *idx, int mode)
 {
+    char sp_buf[10];
+    sprintf(sp_buf, "%%%dc", COLS - 2);
     wattrset(table->tcpscreen, STDATTR);
     if (direction == SCROLLUP) {
 	if (table->lastvisible != table->tail) {
@@ -125,7 +153,7 @@ void scrollupperwin(struct tcptable *table, int direction, int *idx, int mode)
 	    (*idx)++;
 	    wmove(table->tcpscreen, table->imaxy - 1, 0);
 	    scrollok(table->tcpscreen, 0);
-	    wprintw(table->tcpscreen, "%78c", ' ');
+	    wprintw(table->tcpscreen, sp_buf, ' ');
 	    scrollok(table->tcpscreen, 1);
 	    printentry(table, table->lastvisible, *idx, mode);
 	}
@@ -136,7 +164,7 @@ void scrollupperwin(struct tcptable *table, int direction, int *idx, int mode)
 	    table->lastvisible = table->lastvisible->prev_entry;
 	    (*idx)--;
 	    wmove(table->tcpscreen, 0, 0);
-	    wprintw(table->tcpscreen, "%78c", ' ');
+	    wprintw(table->tcpscreen, sp_buf, ' ');
 	    printentry(table, table->firstvisible, *idx, mode);
 	}
     }
@@ -267,14 +295,18 @@ void ipmon(const struct OPTIONS *options, int filtered,
 	   unsigned int tcptm, int facilitytime)
 {
     int logging = options->logging;
-    struct sockaddr_pkt fromaddr;	/* iface info */
-    int fd;				/* raw socket */
-    char tpacket[8192];			/* raw packet data */
-    char *packet = NULL;		/* network packet ptr */
+    struct sockaddr_ll fromaddr;	/* iface info */
+    unsigned short linktype;	/* data link type */
+    int fd;			/* raw socket */
+    int isdn_fd = -1;		/* file descriptor for isdnctrl if any */
+    char tpacket[8194];		/* raw packet data */
+    char aligned_buf[120];	/* aligned buffer */
+    char *packet = NULL;	/* network packet ptr */
     struct iphdr *ippacket;
-    struct tcphdr *transpacket;		/* IP-encapsulated packet */
+    struct tcphdr *transpacket;	/* IP-encapsulated packet */
     unsigned int sport = 0, dport = 0;	/* TCP/UDP port values */
     int firstin = 0;
+    char sp_buf[10];
 
     unsigned int screen_idx = 1;
 
@@ -285,7 +317,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
     unsigned long updtime = 0;
     unsigned long long updtime_usec = 0;
     unsigned long long unow = 0;
-    
+
     WINDOW *statwin;
     PANEL *statpanel;
 
@@ -293,8 +325,6 @@ void ipmon(const struct OPTIONS *options, int filtered,
 
     int curwin = 0;
 
-    int isclosed;
-
     int readlen;
     char ifname[8];
 
@@ -321,13 +351,17 @@ void ipmon(const struct OPTIONS *options, int filtered,
     struct othptable othptbl;
     struct othptabent *othpent;
 
+    struct isdntab isdntable;
+
     int p_sstat = 0, p_dstat = 0;	/* Reverse lookup statuses prior to */
-    					/* reattempt in updateentry(). */
-    int nifltok = 0;			/* Non-IP filter ok */
+    /* reattempt in updateentry(). */
+    int nifltok = 0;		/* Non-IP filter ok */
+
+    int fragment = 0;		/* Set to 1 if not first fragment */
 
     int ch;
     int endloop = 0;
-    char msgstring[82];
+    char msgstring[40];
     int nomem = 0;
 
     int rvnfd = 0;
@@ -337,41 +371,43 @@ void ipmon(const struct OPTIONS *options, int filtered,
      */
 
     if (!facility_active(IPMONIDFILE))
-        mark_facility(IPMONIDFILE, "IP traffic monitor");
+	mark_facility(IPMONIDFILE, "IP traffic monitor");
     else {
-        errbox("IP traffic monitor active in another process", ANYKEY_MSG, &ch);
-        return;
+	errbox("IP traffic monitor active in another process", ANYKEY_MSG, &ch);
+	return;
+    }
+
+    open_socket(&fd);
+
+    if (fd < 0) {
+	return;
     }
-    
     init_tcp_table(&table);
     init_othp_table(&othptbl);
 
-    if (options->revlook)
+    if (options->revlook) {
 	if (checkrvnamed())
 	    open_rvn_socket(&rvnfd);
-	else
-	    rvnfd = 0;
+    } else
+	rvnfd = 0;
 
     if (options->servnames)
 	setservent(1);
 
     ipmonhelp();
-    statwin = newwin(1, 80, LINES - 2, 0);
+    uniq_help(0);
+    statwin = newwin(1, COLS, LINES - 2, 0);
     statpanel = new_panel(statwin);
     wattrset(statwin, FIELDATTR);
     wmove(statwin, 0, 0);
-    wprintw(statwin, "%80c", ' ');
+    sprintf(sp_buf, "%%%dc", COLS);
+    wprintw(statwin, sp_buf, ' ');
     prepare_ip_statwin(statwin);
     show_ip_stats(statwin, 0, 0, 0, 0, 0);
     markactive(curwin, table.borderwin, othptbl.borderwin);
     update_panels();
     doupdate();
 
-    open_socket(&fd, "");
-
-    if (fd < 0)
-	return;
-
     /*
      * Try to open log file if logging activated.  Turn off logging
      * (for this session only) if an error was discovered in opening
@@ -379,19 +415,26 @@ void ipmon(const struct OPTIONS *options, int filtered,
      * situation may be corrected later.
      */
 
+    bzero(&isdntable, sizeof(struct isdntab));
+
     if (logging)
 	opentlog(&logfile, IPMONLOG);
 
     if (logfile == NULL)
 	logging = 0;
 
+    if (logging)
+	signal(SIGUSR1, rotate_ipmon_log);
+
+    rotate_flag = 0;
     writelog(logging, logfile, "******** IP traffic monitor started");
 
     gettimeofday(&tv, NULL);
     starttime = timeint = tv.tv_sec;
 
     while (!endloop) {
-	getpacket(fd, tpacket, &fromaddr, &ch, &readlen, 5, table.tcpscreen);
+	getpacket(fd, tpacket, &fromaddr, &ch, &readlen, ifname, table.tcpscreen);
+
 	if (ch != ERR) {
 	    switch (ch) {
 	    case KEY_UP:
@@ -406,6 +449,22 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		else
 		    scrolllowerwin(&othptbl, SCROLLUP);
 		break;
+	    case KEY_RIGHT:
+	        if (curwin) {
+	            if (othptbl.strindex != VSCRL_OFFSET)
+	                othptbl.strindex = VSCRL_OFFSET;
+	            
+	            refresh_othwindow(&othptbl);
+	        }
+	        break;
+	    case KEY_LEFT:
+	        if (curwin) {
+	            if (othptbl.strindex != 0)
+	                othptbl.strindex = 0;
+	            
+	            refresh_othwindow(&othptbl);
+	        }
+	        break;
 	    case KEY_PPAGE:
 	    case '-':
 		if (!curwin)
@@ -425,11 +484,14 @@ void ipmon(const struct OPTIONS *options, int filtered,
 	    case 'W':
 		curwin = ~curwin;
 		markactive(curwin, table.borderwin, othptbl.borderwin);
+		uniq_help(curwin);
 		break;
 	    case 'm':
 	    case 'M':
-		mode = ~mode;
-		refreshtcpwin(&table, screen_idx, mode);
+	        if (!curwin) {
+		    mode = ~mode;
+		    refreshtcpwin(&table, screen_idx, mode);
+		}
 		break;
 	    case 'Q':
 	    case 'q':
@@ -441,17 +503,20 @@ void ipmon(const struct OPTIONS *options, int filtered,
 	    }
 	}
 	if (readlen > 0) {
-	    strcpy(ifname, fromaddr.spkt_device);
-
 	    if (!iface_supported(ifname))
 		continue;
 
-	    fromaddr.spkt_protocol = htons(fromaddr.spkt_protocol);
+	    isdn_iface_check(&isdn_fd, ifname);
+	    linktype = getlinktype(fromaddr.sll_hatype, ifname,
+				   isdn_fd, &isdntable);
 
-	    if (fromaddr.spkt_protocol != ETH_P_IP) {
-		if ((fromaddr.spkt_protocol == ETH_P_ARP) ||
-		    (fromaddr.spkt_protocol == ETH_P_RARP))
-		    nifltok = othfilterok(ofilter, fromaddr.spkt_protocol,
+	    fromaddr.sll_protocol = htons(fromaddr.sll_protocol);
+	    adjustpacket(tpacket, linktype, &packet, aligned_buf, &readlen);
+
+	    if (fromaddr.sll_protocol != ETH_P_IP) {
+		if ((fromaddr.sll_protocol == ETH_P_ARP) ||
+		    (fromaddr.sll_protocol == ETH_P_RARP))
+		    nifltok = othfilterok(ofilter, fromaddr.sll_protocol,
 					  0, 0, 0, 0);
 		else
 		    nifltok = 1;
@@ -462,20 +527,21 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		 * and nomem not set
 		 */
 
+
 		if ((nifltok) && (!nomem))
 		    othpent = add_othp_entry(&othptbl, &table,
-				    0, 0, NOT_IP, fromaddr.spkt_protocol,
-		    fromaddr.spkt_family, (char *) tpacket, ifname, 0, 0,
-			0, logging, logfile, options->servnames, &nomem);
+				     0, 0, NOT_IP, fromaddr.sll_protocol,
+			     linktype, (char *) tpacket, packet, readlen,
+				       ifname, 0, 0, 0, logging, logfile,
+					  options->servnames, 0, &nomem);
 
-		noniptotal++;
+		noniptotal += readlen;
 		continue;
 	    }
 	    /* 
 	     * move past data link header, if present
 	     */
 
-	    adjustpacket(tpacket, fromaddr.spkt_family, &packet, &readlen);
 
 	    if (packet == NULL)
 		continue;
@@ -539,7 +605,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		}
 		tcpentry = in_table(&table, ippacket->saddr, ippacket->daddr,
 				    ntohs(sport), ntohs(dport), ifname,
-				    &isclosed, tcptm, logging, logfile,
+				    tcptm, logging, logfile,
 				    &nomem);
 
 		/* 
@@ -561,7 +627,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 			 * is not yet closed, or if it is a SYN packet.
 			 */
 
-			if ((!nomem) && ((!isclosed) || (isclosed && transpacket->syn))) {
+			if (!nomem) {
 			    tcpentry = addentry(&table,
 					 (unsigned long) ippacket->saddr,
 					 (unsigned long) ippacket->daddr,
@@ -569,16 +635,6 @@ void ipmon(const struct OPTIONS *options, int filtered,
 						options->revlook, rvnfd,
 					     options->servnames, &nomem);
 
-			    if ((tcpentry != NULL) && (logging)) {
-				sprintf(msgstring, "TCP: %s:%s to %s:%s first packet received ",
-				     tcpentry->s_fqdn, tcpentry->s_sname,
-				    tcpentry->d_fqdn, tcpentry->d_sname);
-
-				if (transpacket->syn)
-				    strcat(msgstring, "(SYN)");
-
-				writelog(logging, logfile, msgstring);
-			    }
 			    if (tcpentry != NULL)
 				printentry(&table, tcpentry->oth_connection,
 					   screen_idx, mode);
@@ -593,6 +649,14 @@ void ipmon(const struct OPTIONS *options, int filtered,
 
 		if (tcpentry != NULL) {
 
+		    if ((tcpentry->pcount == 0) && (logging)) {
+			strcpy(msgstring, "first packet");
+			if (transpacket->syn)
+			    strcat(msgstring, " (SYN)");
+
+			writetcplog(logging, logfile,
+				    tcpentry, nbplen, msgstring);
+		    }
 		    /* 
 		     * Don't bother updating the entry if the connection
 		     * has been previously reset.  (Does this really
@@ -634,8 +698,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		    }
 		}
 	    } else {		/* now for the other IP protocols */
-		if ((ntohs(ippacket->frag_off) & 0x1fff) != 0)
-		    continue;	/* Just interpret first fragment */
+		fragment = ((ntohs(ippacket->frag_off) & 0x1fff) != 0);
 
 		if (ippacket->protocol == IPPROTO_UDP)
 		    udptotal += nbplen;
@@ -649,7 +712,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 
 		    if (((struct icmphdr *) transpacket)->type == ICMP_DEST_UNREACH)
 			process_dest_unreach(&table, (char *) transpacket,
-			                     ifname, &nomem);
+					     ifname, &nomem);
 
 		    icmptotal += nbplen;
 		} else
@@ -667,9 +730,11 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		    othpent = add_othp_entry(&othptbl, &table,
 					ippacket->saddr, ippacket->daddr,
 					     IS_IP, ippacket->protocol, 0,
-					     (char *) transpacket, ifname,
-			options->revlook, rvnfd, tcptm, logging, logfile,
-					     options->servnames, &nomem);
+				     (char *) transpacket, (char *) NULL,
+					nbplen, ifname, options->revlook,
+					  rvnfd, tcptm, logging, logfile,
+					     options->servnames, fragment,
+					     &nomem);
 		}
 	    }
 
@@ -679,21 +744,28 @@ void ipmon(const struct OPTIONS *options, int filtered,
 	gettimeofday(&tv, NULL);
 	now = tv.tv_sec;
 	unow = tv.tv_sec * 1e+06 + tv.tv_usec;
-	
+
 	if (now - timeint >= 5) {
 	    printelapsedtime(starttime, now, othptbl.obmaxy - 1, 15, othptbl.borderwin);
 	    timeint = now;
 	}
 	if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-	   ((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
+	    ((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
 	    update_panels();
 	    doupdate();
 	    updtime = now;
 	    updtime_usec = unow;
 	}
-	
 	if ((facilitytime != 0) && (((now - starttime) / 60) >= facilitytime))
 	    endloop = 1;
+
+	if ((rotate_flag == 1) && (logging)) {
+	    announce_rotate_prepare(logfile);
+	    write_tcp_unclosed(logging, logfile, &table);
+	    rotate_logfile(&logfile, target_logname);
+	    announce_rotate_complete(logfile);
+	    rotate_flag = 0;
+	}
     }
 
     killrvnamed();
@@ -702,7 +774,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 	endservent();
 
     close_rvn_socket(rvnfd);
-
+    close(isdn_fd);
     del_panel(table.tcppanel);
     del_panel(table.borderpanel);
     del_panel(othptbl.othppanel);
@@ -719,10 +791,13 @@ void ipmon(const struct OPTIONS *options, int filtered,
     destroytcptable(&table);
     destroyothptable(&othptbl);
     destroyfraglist();
+    destroy_isdn_table(&isdntable);
     writelog(logging, logfile, "******** IP traffic monitor stopped\n");
+    unlink(IPMONIDFILE);
     if (logfile != NULL)
 	fclose(logfile);
-	
-    unlink(IPMONIDFILE);
+
+    signal(SIGUSR1, SIG_DFL);
+
     return;
 }
