@@ -184,7 +184,7 @@ void operatemenu(struct MENU *menu, int *position, int *aborted)
     *aborted = 0;
     menumoveto(menu, &itemptr, row);
 
-    menu->descwin = newwin(1, 80, LINES - 2, 0);
+    menu->descwin = newwin(1, COLS, LINES - 2, 0);
     menu->descpanel = new_panel(menu->descwin);
 
     do {
@@ -236,7 +236,7 @@ void operatemenu(struct MENU *menu, int *position, int *aborted)
 	    break;
 	    /* case 27: exitloop = 1;*aborted = 1;row=menu->itemcount;break; */
 	case '^':
-	    break;                 /* ignore caret key */
+	    break;		/* ignore caret key */
 	default:
 	    keyptr = strchr(menu->shortcuts, toupper(ch));
 	    if ((keyptr != NULL) && keyptr - menu->shortcuts < menu->itemcount) {
