@@ -22,7 +22,7 @@ details.
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <net/if_arp.h>
+#include "links.h"
 #include "serv.h"
 #include "options.h"
 #include "deskman.h"
@@ -41,12 +41,13 @@ extern void srpromisc(int mode);
 
 void makeoptionmenu(struct MENU *menu)
 {
-    initmenu(menu, 19, 30, 3, 10);
+    initmenu(menu, 20, 37, (LINES - 20) / 2, (COLS - 37) / 4);
     additem(menu, " ^R^everse DNS lookups", "Toggles resolution of IP addresses into host names");
     additem(menu, " TCP/UDP ^s^ervice names", "Displays TCP/UDP service names instead of numeric ports");
     additem(menu, " ^P^romiscuous operation", "Toggles capture of all packets by LAN interfaces");
     additem(menu, " ^C^olor", "Turns color on or off (restart to effect change)");
     additem(menu, " ^L^ogging", "Toggles logging of traffic to a data file");
+    additem(menu, " Acti^v^ity mode", "Toggles activity indicators between kbits/s and kbytes/s");
     additem(menu, NULL, NULL);
     additem(menu, " TCP ^t^imeout...", "Determines how long TCP entries may remain idle");
     additem(menu, " Logging ^i^nterval...", "Sets the time between loggings for interface, host, and service stats");
@@ -55,18 +56,18 @@ void makeoptionmenu(struct MENU *menu)
     additem(menu, " ^A^dditional ports...", "Allows you to add port numbers higher than 1023 for the service stats");
     additem(menu, " ^D^elete port/range...", "Deletes a port or range of ports earlier added");
     additem(menu, NULL, NULL);
-    additem(menu, " ^E^thernet host descriptions", "Manages descriptions for Ethernet addresses");
+    additem(menu, " ^E^thernet/PLIP host descriptions", "Manages descriptions for Ethernet/PLIP addresses");
     additem(menu, " ^F^DDI host descriptions", "Manages descriptions for FDDI addresses");
     additem(menu, NULL, NULL);
     additem(menu, " E^x^it configuration", "Returns to main menu");
 }
 
-void printoptonoff(unsigned int option, WINDOW *win)
+void printoptonoff(unsigned int option, WINDOW * win)
 {
     if (option)
-        wprintw(win, " On");
+	wprintw(win, " On");
     else
-        wprintw(win, "Off");
+	wprintw(win, "Off");
 }
 
 void indicatesetting(int row, struct OPTIONS *options, WINDOW * win)
@@ -90,6 +91,12 @@ void indicatesetting(int row, struct OPTIONS *options, WINDOW * win)
     case 5:
 	printoptonoff(options->logging, win);
 	break;
+    case 6:
+	wmove(win, row, 20);
+	if (options->actmode == KBITS)
+	    wprintw(win, " kbits/s");
+	else
+	    wprintw(win, "kbytes/s");
     }
 }
 
@@ -120,6 +127,7 @@ void setdefaultopts(struct OPTIONS *options)
     options->servnames = 0;
     options->color = 1;
     options->logging = 0;
+    options->actmode = KBITS;
     options->timeout = 15;
     options->logspan = 3600;
     options->updrate = 0;
@@ -144,11 +152,11 @@ void loadoptions(struct OPTIONS *options)
 void updatetimes(struct OPTIONS *options, WINDOW * win)
 {
     wattrset(win, HIGHATTR);
-    wmove(win, 7, 20);
-    wprintw(win, "%3u mins", options->timeout);
     wmove(win, 8, 20);
-    wprintw(win, "%3u mins", options->logspan / 60);
+    wprintw(win, "%3u mins", options->timeout);
     wmove(win, 9, 20);
+    wprintw(win, "%3u mins", options->logspan / 60);
+    wmove(win, 10, 20);
     wprintw(win, "%3u secs", options->updrate);
 }
 
@@ -156,7 +164,7 @@ void showoptions(struct OPTIONS *options, WINDOW * win)
 {
     int i;
 
-    for (i = 1; i <= 5; i++)
+    for (i = 1; i <= 6; i++)
 	indicatesetting(i, options, win);
 
     updatetimes(options, win);
@@ -170,7 +178,7 @@ void settimeout(unsigned int *value, const char *units, int allow_zero, int *abo
     int resp;
     unsigned int tmval = 0;
 
-    dlgwin = newwin(7, 40, 10, 10);
+    dlgwin = newwin(7, 40, (LINES - 7) / 2, (COLS - 40) / 4);
     dlgpanel = new_panel(dlgwin);
 
     wattrset(dlgwin, BOXATTR);
@@ -183,7 +191,7 @@ void settimeout(unsigned int *value, const char *units, int allow_zero, int *abo
     wmove(dlgwin, 5, 2);
     stdkeyhelp(dlgwin);
 
-    initfields(&field, 1, 10, 13, 12);
+    initfields(&field, 1, 10, (LINES - 7) / 2 + 3, (COLS - 40) / 4 + 2);
     addfield(&field, 3, 0, 0, "");
 
     do {
@@ -212,25 +220,24 @@ void setoptions(struct OPTIONS *options, struct porttab **ports, int is_first_in
     int row = 1;
     int aborted;
     int resp;
-    
+
     struct MENU menu;
     WINDOW *statwin;
     PANEL *statpanel;
 
     if (!is_first_instance) {
-        errbox("Only the first instance of IPTraf can configure", ANYKEY_MSG, &resp);
-        return;
+	errbox("Only the first instance of IPTraf can configure", ANYKEY_MSG, &resp);
+	return;
     }
-
     makeoptionmenu(&menu);
 
-    statwin = newwin(11, 30, 6, 40);
+    statwin = newwin(12, 30, (LINES - 20) / 2, (COLS - 37) / 4 + 37);
     statpanel = new_panel(statwin);
 
     wattrset(statwin, BOXATTR);
     colorwin(statwin);
     box(statwin, ACS_VLINE, ACS_HLINE);
-    wmove(statwin, 6, 1);
+    wmove(statwin, 7, 1);
     whline(statwin, ACS_HLINE, 28);
     wmove(statwin, 0, 1);
     wprintw(statwin, " Current Settings ");
@@ -245,11 +252,13 @@ void setoptions(struct OPTIONS *options, struct porttab **ports, int is_first_in
     wprintw(statwin, "Color: ");
     wmove(statwin, 5, 2);
     wprintw(statwin, "Logging: ");
-    wmove(statwin, 7, 2);
-    wprintw(statwin, "TCP timeout:");
+    wmove(statwin, 6, 2);
+    wprintw(statwin, "Activity mode: ");
     wmove(statwin, 8, 2);
-    wprintw(statwin, "Log interval:");
+    wprintw(statwin, "TCP timeout:");
     wmove(statwin, 9, 2);
+    wprintw(statwin, "Log interval:");
+    wmove(statwin, 10, 2);
     wprintw(statwin, "Update interval:");
     showoptions(options, statwin);
 
@@ -274,39 +283,42 @@ void setoptions(struct OPTIONS *options, struct porttab **ports, int is_first_in
 	case 5:
 	    options->logging = ~(options->logging);
 	    break;
-	case 7:
-	    settimeout(&(options->timeout), "minutes", DONT_ALLOW_ZERO, &aborted);
-	    if (!aborted)
-	        updatetimes(options, statwin);
+	case 6:
+	    options->actmode = ~(options->actmode);
 	    break;
 	case 8:
-	    settimeout((unsigned int *) &(options->logspan), "minutes", DONT_ALLOW_ZERO, &aborted);
-	    if (!aborted) {
-	        options->logspan = options->logspan * 60;
-	        updatetimes(options, statwin);
-	    }
+	    settimeout(&(options->timeout), "minutes", DONT_ALLOW_ZERO, &aborted);
+	    if (!aborted)
+		updatetimes(options, statwin);
 	    break;
 	case 9:
+	    settimeout((unsigned int *) &(options->logspan), "minutes", DONT_ALLOW_ZERO, &aborted);
+	    if (!aborted) {
+		options->logspan = options->logspan * 60;
+		updatetimes(options, statwin);
+	    }
+	    break;
+	case 10:
 	    settimeout(&options->updrate, "seconds", ALLOW_ZERO, &aborted);
 	    if (!aborted)
-	        updatetimes(options, statwin);
-	    break;
-	case 11:
-	    addmoreports(ports);
+		updatetimes(options, statwin);
 	    break;
 	case 12:
+	    addmoreports(ports);
+	    break;
+	case 13:
 	    removeaport(ports);
 	    break;
-	case 14:
-	    ethdescmgr(ARPHRD_ETHER);
-	    break;
 	case 15:
-	    ethdescmgr(ARPHRD_FDDI);
+	    ethdescmgr(LINK_ETHERNET);
+	    break;
+	case 16:
+	    ethdescmgr(LINK_FDDI);
 	    break;
 	}
 
 	indicatesetting(row, options, statwin);
-    } while (row != 17);
+    } while (row != 18);
 
     destroymenu(&menu);
     del_panel(statpanel);
