@@ -41,7 +41,7 @@ details.
 
 void makeothfiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 11, 15, 8, 15);
+    initmenu(menu, 11, 15, (LINES - 11) / 2, (COLS - 30) / 2 - 5);
     additem(menu, " ^U^DP...", "User Datagram Protocol");
     additem(menu, " ^I^CMP", "Internet Control Message Protocol");
     additem(menu, " ^O^SPF", "Open Shortest Path First");
@@ -211,7 +211,7 @@ void othfilterselect(struct othpoptions *filter)
     int row;
     int aborted;
 
-    statwin = newwin(11, 23, 8, 30);
+    statwin = newwin(11, 23, (LINES - 11) / 2, (COLS - 30) / 2 + 10);
     statpanel = new_panel(statwin);
     wattrset(statwin, BOXATTR);
     colorwin(statwin);
@@ -249,8 +249,8 @@ void setodefaults(struct othpoptions *filter)
 {
     bzero(filter, sizeof(struct othpoptions));
     filter->filtercode = 1;
-    filter->udp = filter->icmp = filter->ospf = (__u16) ~ 0;
-    filter->arp = filter->rarp = (__u16) ~ 0;
+    filter->udp = filter->icmp = filter->ospf = (u_int16_t) ~ 0;
+    filter->arp = filter->rarp = (u_int16_t) ~ 0;
 }
 
 void loadothfilter(struct othpoptions *filter)
@@ -283,13 +283,13 @@ void saveothfilter(struct othpoptions *filter)
     int resp;
 
     if (!facility_active(OTHFLTIDFILE))
-        mark_facility(OTHFLTIDFILE, "Non-TCP filter change");
+	mark_facility(OTHFLTIDFILE, "Non-TCP filter change");
     else {
-        errbox("Non-TCP data file currently in use; try again later",
-               ANYKEY_MSG, &resp);
-        return;
+	errbox("Non-TCP data file currently in use; try again later",
+	       ANYKEY_MSG, &resp);
+	return;
     }
-    
+
     pfd = open(OTHFLNAME, O_CREAT | O_TRUNC | O_WRONLY, S_IRUSR | S_IWUSR);
     bw = write(pfd, filter, sizeof(struct othpoptions));
     if (bw < 1)
