@@ -9,16 +9,13 @@ Copyright (c) Gerard Paul Java 1997
 
 #include <sys/types.h>
 #include <netinet/udp.h>
-#ifndef _I386_TYPES_H
-#include <asm/types.h>
-#endif
 
 struct othpoptions {
     char filename[40];		/* these are for the UDP filters */
     struct filterlist fl;
     unsigned int filtercode;
 
-    __u16 udp:1, icmp:1, ospf:1, igp:1, igmp:1, igrp:1, arp:1, rarp:1,
+    u_int16_t udp:1, icmp:1, ospf:1, igp:1, igmp:1, igrp:1, arp:1, rarp:1,
      padding:8;
 };
 
