@@ -6,6 +6,9 @@ promisc.c	- handles the promiscuous mode flag for the Ethernet
 Written by Gerard Paul Java
 Copyright (c) Gerard Paul Java 1997, 1998
 
+This module sets an interface's promiscuous flag using the standard ioctl()
+SIOCSIFFLAGS command.
+
 This software is open source; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
@@ -30,8 +33,8 @@ details.
 #include "ifstats.h"
 #include "error.h"
 
-extern initiflist(struct iflist **list);
-extern destroyiflist(struct iflist *list);
+extern void initiflist(struct iflist **list);
+extern void destroyiflist(struct iflist *list);
 
 void srpromisc(int mode)
 {
@@ -46,7 +49,7 @@ void srpromisc(int mode)
 
     initiflist(&list);
     ptmp = list;
-    fd = socket(PF_INET, SOCK_PACKET, htons(ETH_P_ALL));
+    fd = socket(PF_INET, SOCK_DGRAM, 0);
 
     if (fd < 0) {
 	errbox("Unable to open socket for flag change", ANYKEY_MSG, &response);
