@@ -3,7 +3,7 @@
 rvnamed		- reverse DNS lookup daemon for the IPTraf network
 		  statistics utility.
 
-Version 1.2.0                      Parallel with IPTraf 1.2 and 1.3
+Version 2.0.0                      Parallel with IPTraf 2.0
 
 Written by Gerard Paul Java
 Copyright (c) Gerard Paul Java 1998
@@ -48,6 +48,8 @@ details.
 #include "rvnamed.h"
 #include "dirs.h"
 
+#define NUM_CACHE_ENTRIES 2048
+
 struct hosts {
     unsigned long addr;
     char fqdn[45];
@@ -85,8 +87,8 @@ void process_rvn_packet(struct rvn *rvnpacket)
     if (he == NULL)
 	strcpy(rvnpacket->fqdn, inet_ntoa(rvnpacket->saddr));
     else {
-	bzero(rvnpacket->fqdn, 40);
-	strncpy(rvnpacket->fqdn, he->h_name, 30);
+	bzero(rvnpacket->fqdn, 45);
+	strncpy(rvnpacket->fqdn, he->h_name, 44);
     }
 
     ccsa.sun_family = AF_UNIX;
@@ -150,12 +152,12 @@ void writervnlog(FILE * fd, char *msg)
     fprintf(fd, "%s: %s\n", atime, msg);
 }
 
-void main(void)
+int main(void)
 {
     int cfd;
     int ifd;
 
-    struct hosts hostlist[512];
+    struct hosts hostlist[NUM_CACHE_ENTRIES];
     unsigned int hostindex = 0;
     unsigned int lastfree = 0;
     unsigned int hi = 0;
@@ -273,12 +275,12 @@ void main(void)
 		    bzero(&(hostlist[hi]), sizeof(struct hosts));
 		    hi = hostindex;
 		    hostindex++;
-		    if (hostindex == 512)
+		    if (hostindex == NUM_CACHE_ENTRIES)
 			hostindex = 0;
 
 		    hostlist[hi].addr = rvnpacket.saddr.s_addr;
 		}
-		strncpy(hostlist[hi].fqdn, rvnpacket.fqdn, 30);
+		strncpy(hostlist[hi].fqdn, rvnpacket.fqdn, 44);
 
 		hostlist[hi].ready = RESOLVED;
 	    }
@@ -318,8 +320,8 @@ void main(void)
 		    readyidx = name_resolved(&rvnpacket, hostlist, lastfree);
 		    if (readyidx >= 0) {
 			rvnpacket.type = RVN_REPLY;
-			bzero(rvnpacket.fqdn, 30);
-			strncpy(rvnpacket.fqdn, hostlist[readyidx].fqdn, 15);
+			bzero(rvnpacket.fqdn, 45);
+			strncpy(rvnpacket.fqdn, hostlist[readyidx].fqdn, 44);
 			rvnpacket.ready = RESOLVED;
 
 			fromaddr.sun_family = AF_UNIX;
@@ -340,10 +342,10 @@ void main(void)
 
 			    hostindex++;
 
-			    if (hostindex == 512)
+			    if (hostindex == NUM_CACHE_ENTRIES)
 				hostindex = 0;
 
-			    if (lastfree < 512)
+			    if (lastfree < NUM_CACHE_ENTRIES)
 				lastfree++;
 
 			    fflush(logfile);	/* flush all data prior */
@@ -361,7 +363,7 @@ void main(void)
 			    }
 			}
 			rvnpacket.type = RVN_REPLY;
-			bzero(rvnpacket.fqdn, 30);
+			bzero(rvnpacket.fqdn, 45);
 			strcpy(rvnpacket.fqdn, inet_ntoa(rvnpacket.saddr));
 			rvnpacket.ready = RESOLVING;
 
