@@ -21,6 +21,7 @@ details.
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <signal.h>
 #include <sys/time.h>
 #include <netdb.h>
 #include <sys/types.h>
@@ -35,6 +36,7 @@ details.
 #include "dirs.h"
 #include "ipcsum.h"
 #include "deskman.h"
+#include "isdntab.h"
 #include "packet.h"
 #include "ipfrag.h"
 #include "ifaces.h"
@@ -49,39 +51,52 @@ details.
 #include "options.h"
 #include "instances.h"
 #include "packet.h"
+#include "logvars.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
 
 extern void writeutslog(struct portlistent *list, unsigned long nsecs, FILE * logfile);
 
+/*
+ * SIGUSR1 logfile rotation signal handler
+ */
+
+void rotate_serv_log()
+{
+    rotate_flag = 1;
+    strcpy(target_logname, TCPUDPLOG);
+    signal(SIGUSR1, rotate_serv_log);
+}
+
 void initportlist(struct portlist *list)
 {
+    float screen_scale = ((float) COLS / 80 + 1) / 2;
     list->head = list->tail = NULL;
     list->firstvisible = list->lastvisible = NULL;
     list->count = 0;
 
-    list->borderwin = newwin(LINES - 2, 80, 1, 0);
+    list->borderwin = newwin(LINES - 2, COLS, 1, 0);
     list->borderpanel = new_panel(list->borderwin);
     wattrset(list->borderwin, BOXATTR);
     box(list->borderwin, ACS_VLINE, ACS_HLINE);
 
-    wmove(list->borderwin, 0, 1);
+    wmove(list->borderwin, 0, 1 * screen_scale);
     wprintw(list->borderwin, " Proto/Port ");
-    wmove(list->borderwin, 0, 22);
+    wmove(list->borderwin, 0, 22 * screen_scale);
     wprintw(list->borderwin, " Pkts ");
-    wmove(list->borderwin, 0, 31);
+    wmove(list->borderwin, 0, 31 * screen_scale);
     wprintw(list->borderwin, " Bytes ");
-    wmove(list->borderwin, 0, 40);
+    wmove(list->borderwin, 0, 40 * screen_scale);
     wprintw(list->borderwin, " PktsTo ");
-    wmove(list->borderwin, 0, 49);
+    wmove(list->borderwin, 0, 49 * screen_scale);
     wprintw(list->borderwin, " BytesTo ");
-    wmove(list->borderwin, 0, 58);
+    wmove(list->borderwin, 0, 58 * screen_scale);
     wprintw(list->borderwin, " PktsFrom ");
-    wmove(list->borderwin, 0, 67);
+    wmove(list->borderwin, 0, 67 * screen_scale);
     wprintw(list->borderwin, " BytesFrom ");
 
-    list->win = newwin(LINES - 4, 78, 2, 1);
+    list->win = newwin(LINES - 4, COLS - 2, 2, 1);
     list->panel = new_panel(list->win);
 
     stdwinset(list->win);
@@ -147,7 +162,7 @@ int portinlist(struct porttab *table, unsigned int port)
 
     while (ptmp != NULL) {
 	if (((ptmp->port_max == 0) && (ptmp->port_min == port)) ||
-	   ((port >= ptmp->port_min) && (port <= ptmp->port_max)))
+	    ((port >= ptmp->port_min) && (port <= ptmp->port_max)))
 	    return 1;
 
 	ptmp = ptmp->next_entry;
@@ -180,6 +195,7 @@ void printportent(struct portlist *list, struct portlistent *entry,
 		  unsigned int idx)
 {
     unsigned int target_row;
+    float screen_scale = ((float) COLS / 80 + 1) / 2;
 
     if ((entry->idx < idx) || (entry->idx > idx + (LINES - 5)))
 	return;
@@ -196,17 +212,17 @@ void printportent(struct portlist *list, struct portlistent *entry,
     }
     wprintw(list->win, "/%s: ", entry->servname);
     wattrset(list->win, HIGHATTR);
-    wmove(list->win, target_row, 17);
+    wmove(list->win, target_row, 17 * screen_scale);
     printlargenum(entry->count, list->win);
-    wmove(list->win, target_row, 27);
+    wmove(list->win, target_row, 27 * screen_scale);
     printlargenum(entry->bcount, list->win);
-    wmove(list->win, target_row, 37);
+    wmove(list->win, target_row, 37 * screen_scale);
     printlargenum(entry->icount, list->win);
-    wmove(list->win, target_row, 47);
+    wmove(list->win, target_row, 47 * screen_scale);
     printlargenum(entry->ibcount, list->win);
-    wmove(list->win, target_row, 57);
+    wmove(list->win, target_row, 57 * screen_scale);
     printlargenum(entry->ocount, list->win);
-    wmove(list->win, target_row, 67);
+    wmove(list->win, target_row, 67 * screen_scale);
     printlargenum(entry->obcount, list->win);
 }
 
@@ -260,6 +276,8 @@ void printport(struct portlist *list, unsigned int protocol, unsigned int port,
 
 void scrollservwin(struct portlist *table, int direction, int *idx)
 {
+    char sp_buf[10];
+    sprintf(sp_buf, "%%%dc", COLS - 2);
     wattrset(table->win, STDATTR);
     if (direction == SCROLLUP) {
 	if (table->lastvisible != table->tail) {
@@ -269,7 +287,7 @@ void scrollservwin(struct portlist *table, int direction, int *idx)
 	    (*idx)++;
 	    wmove(table->win, LINES - 5, 0);
 	    scrollok(table->win, 0);
-	    wprintw(table->win, "%78c", ' ');
+	    wprintw(table->win, sp_buf, ' ');
 	    scrollok(table->win, 1);
 	    printportent(table, table->lastvisible, *idx);
 	}
@@ -280,7 +298,7 @@ void scrollservwin(struct portlist *table, int direction, int *idx)
 	    table->firstvisible = table->firstvisible->prev_entry;
 	    (*idx)--;
 	    wmove(table->win, 0, 0);
-	    wprintw(table->win, "%78c", ' ');
+	    wprintw(table->win, sp_buf, ' ');
 	    printportent(table, table->firstvisible, *idx);
 	}
     }
@@ -306,32 +324,38 @@ void pageservwin(struct portlist *table, int direction, int *idx)
 /*
  * The TCP/UDP service monitor
  */
- 
+
 void servmon(char *ifname, struct porttab *ports,
-             const struct OPTIONS *options, int facilitytime)
+	     const struct OPTIONS *options, int facilitytime)
 {
     int logging = options->logging;
     int fd;
+    int isdn_fd = -1;
+
     char buf[8192];
+    char aligned_buf[120];
     char *ipacket;
     char *tpacket;
 
     unsigned int iphlen;
-    struct sockaddr_pkt fromaddr;
+    struct sockaddr_ll fromaddr;
+    unsigned short linktype;
     int br;
     int exitloop = 0;
 
+    char iface[8];
     unsigned int idx = 1;
 
     unsigned int sport = 0;
     unsigned int dport = 0;
-    unsigned int firstin;    
+    unsigned int firstin;
 
     struct timeval tv;
     unsigned long starttime, startlog, timeint;
-    unsigned long now, unow;
+    unsigned long now;
+    unsigned long long unow;
     unsigned long updtime = 0;
-    unsigned long updtime_usec = 0;
+    unsigned long long updtime_usec = 0;
 
     int csum;
     int ck;
@@ -340,26 +364,37 @@ void servmon(char *ifname, struct porttab *ports,
 
     struct portlist list;
 
+    struct isdntab isdntable;
+
     FILE *logfile = NULL;
 
     /*
      * Mark this facility
      */
-     
+
     if (!facility_active(TCPUDPIDFILE))
-        mark_facility(TCPUDPIDFILE, "TCP/UDP monitor");
+	mark_facility(TCPUDPIDFILE, "TCP/UDP monitor");
     else {
-        errbox("TCP/UDP monitor already active in another process", ANYKEY_MSG, &ch);
+	errbox("TCP/UDP monitor already active in another process", ANYKEY_MSG, &ch);
 	return;
-    }         
-
-    if (!iface_supported(ifname)) {
-        err_iface_unsupported();
-        unlink(TCPUDPIDFILE);
-        return;
     }
-        
 
+    open_socket(&fd);
+
+    if (fd < 0) {
+	unlink(TCPUDPIDFILE);
+	return;
+    }
+    if (!iface_supported(ifname)) {
+	err_iface_unsupported();
+	unlink(TCPUDPIDFILE);
+	return;
+    }
+    if (!iface_up(ifname)) {
+	err_iface_down();
+	unlink(TCPUDPIDFILE);
+	return;
+    }
     initportlist(&list);
 
     move(LINES - 1, 1);
@@ -378,18 +413,19 @@ void servmon(char *ifname, struct porttab *ports,
 	if (logfile == NULL)
 	    logging = 0;
     }
+    if (logging)
+	signal(SIGUSR1, rotate_serv_log);
+
+    rotate_flag = 0;
     writelog(logging, logfile, "******** TCP/UDP service monitor started");
 
-    open_socket(&fd, ifname);
-
-    if (fd < 0)
-	return;
+    bzero(&isdntable, sizeof(struct isdntab));
 
     gettimeofday(&tv, NULL);
     starttime = startlog = timeint = tv.tv_sec;
 
     do {
-	getpacket(fd, buf, &fromaddr, &ch, &br, 5, list.win);
+	getpacket(fd, buf, &fromaddr, &ch, &br, iface, list.win);
 
 	if (ch != ERR)
 	    switch (ch) {
@@ -411,14 +447,19 @@ void servmon(char *ifname, struct porttab *ports,
 	    case 'Q':
 	    case 'x':
 	    case 'X':
+	    case 27:
 	    case 24:
 		exitloop = 1;
 	    }
 	if (br > 0) {
-	    if ((fromaddr.spkt_protocol == ntohs(ETH_P_IP)) &&
-	       ((strcmp(fromaddr.spkt_device, ifname) == 0))) {
-		adjustpacket(buf, fromaddr.spkt_family, &ipacket,
-			     &br);
+	    if ((fromaddr.sll_protocol == ntohs(ETH_P_IP)) &&
+		((strcmp(iface, ifname) == 0))) {
+
+		isdn_iface_check(&isdn_fd, iface);
+		linktype = getlinktype(fromaddr.sll_hatype, iface,
+				       isdn_fd, &isdntable);
+
+		adjustpacket(buf, linktype, &ipacket, aligned_buf, &br);
 
 		if (ipacket == NULL)
 		    continue;
@@ -444,14 +485,14 @@ void servmon(char *ifname, struct porttab *ports,
 		    tpacket = ipacket + iphlen;
 
 		    if (((struct iphdr *) ipacket)->protocol == IPPROTO_TCP) {
-		        sport = ntohs(((struct tcphdr *) tpacket)->source);
-		        dport = ntohs(((struct tcphdr *) tpacket)->dest);
+			sport = ntohs(((struct tcphdr *) tpacket)->source);
+			dport = ntohs(((struct tcphdr *) tpacket)->dest);
 		    } else if (((struct iphdr *) ipacket)->protocol == IPPROTO_UDP) {
-		        sport = ntohs(((struct udphdr *) tpacket)->source);
-		        dport = ntohs(((struct udphdr *) tpacket)->dest);
+			sport = ntohs(((struct udphdr *) tpacket)->source);
+			dport = ntohs(((struct udphdr *) tpacket)->dest);
 		    }
 		}
-		
+
 		if ((((struct iphdr *) ipacket)->protocol == IPPROTO_TCP) ||
 		 (((struct iphdr *) ipacket)->protocol == IPPROTO_UDP)) {
 		    printport(&list, ((struct iphdr *) ipacket)->protocol,
@@ -465,7 +506,7 @@ void servmon(char *ifname, struct porttab *ports,
 	}
 	gettimeofday(&tv, NULL);
 	now = tv.tv_sec;
-	unow = tv.tv_usec;
+	unow = tv.tv_sec * 1e+6 + tv.tv_usec;
 
 	if (now - timeint >= 5) {
 	    printelapsedtime(starttime, now, LINES - 3, 20, list.borderwin);
@@ -476,17 +517,20 @@ void servmon(char *ifname, struct porttab *ports,
 	    startlog = now;
 	}
 	if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-	   ((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
+	    ((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
 	    update_panels();
 	    doupdate();
 	    updtime = now;
 	    updtime_usec = unow;
 	}
+	check_rotate_flag(&logfile, logging);
+
 	if ((facilitytime != 0) && (((now - starttime) / 60) >= facilitytime))
 	    exitloop = 1;
     } while (!exitloop);
 
     if (logging) {
+	signal(SIGUSR1, SIG_DFL);
 	writeutslog(list.head, time((time_t *) NULL) - starttime, logfile);
 	writelog(logging, logfile, "******** TCP/UDP service monitor stopped");
 	fclose(logfile);
@@ -503,6 +547,7 @@ void servmon(char *ifname, struct porttab *ports,
     doupdate();
     destroyportlist(&list);
     destroyfraglist();
+    destroy_isdn_table(&isdntable);
 }
 
 void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
@@ -514,10 +559,11 @@ void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
 
     struct FIELDLIST list;
 
-    bw = newwin(14, 50, 5, 5);
+    /* TODO? : Adjust values for !80 column displays */
+    bw = newwin(14, 50, (LINES - 14) / 2, (COLS - 50) / 2 - 10);
     bp = new_panel(bw);
 
-    win = newwin(12, 48, 6, 6);
+    win = newwin(12, 48, (LINES - 14) / 2 + 1, (COLS - 50) / 2 - 9);
     panel = new_panel(win);
 
     wattrset(bw, BOXATTR);
@@ -547,11 +593,11 @@ void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
     tabkeyhelp(win);
     stdkeyhelp(win);
 
-    initfields(&list, 1, 20, 15, 8);
+    initfields(&list, 1, 20, (LINES - 14) / 2 + 10, (COLS - 50) / 2 - 8);
     wattrset(list.fieldwin, STDATTR);
     wmove(list.fieldwin, 0, 6);
     wprintw(list.fieldwin, "to");
-    
+
     addfield(&list, 5, 0, 0, "");
     addfield(&list, 5, 0, 9, "");
 
@@ -561,7 +607,6 @@ void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
 	*port_min = atoi(list.list->buf);
 	*port_max = atoi(list.list->nextfield->buf);
     }
-
     del_panel(bp);
     delwin(bw);
     del_panel(panel);
@@ -585,14 +630,13 @@ void saveportlist(struct porttab *table)
     while (ptmp != NULL) {
 	bw = write(fd, &(ptmp->port_min), sizeof(unsigned int));
 	bw = write(fd, &(ptmp->port_max), sizeof(unsigned int));
-	
+
 	if (bw < 0) {
 	    errbox("Unable to write port/range entry", ANYKEY_MSG, &resp);
 	    destroyporttab(table);
 	    close(fd);
 	    return;
 	}
-	
 	ptmp = ptmp->next_entry;
     }
 
@@ -689,11 +733,11 @@ void loadaddports(struct porttab **table)
     close(fd);
 }
 
-void displayportentry(struct porttab *ptmp, WINDOW *win)
+void displayportentry(struct porttab *ptmp, WINDOW * win)
 {
     wprintw(win, "%u", ptmp->port_min);
     if (ptmp->port_max != 0)
-        wprintw(win, " to %u", ptmp->port_max);	    
+	wprintw(win, " to %u", ptmp->port_max);
 }
 
 void displayports(struct porttab **table, WINDOW * win)
@@ -704,7 +748,7 @@ void displayports(struct porttab **table, WINDOW * win)
     do {
 	wmove(win, i, 2);
 	displayportentry(ptmp, win);
-		    
+
 	i++;
 	ptmp = ptmp->next_entry;
     } while ((i < 18) && (ptmp != NULL));
@@ -794,14 +838,14 @@ void selectport(struct porttab **table, struct porttab **node, int *aborted)
 	errbox("No custom ports", ANYKEY_MSG, &resp);
 	return;
     }
-    bw = newwin(20, 24, 2, 20);
+    bw = newwin(20, 24, (LINES - 20) / 2 - 2, (COLS - 24) / 2 + 10);
     bp = new_panel(bw);
     wattrset(bw, BOXATTR);
     box(bw, ACS_VLINE, ACS_HLINE);
     wmove(bw, 0, 1);
     wprintw(bw, " Select port entry ");
-    
-    win = newwin(18, 22, 3, 21);
+
+    win = newwin(18, 22, (LINES - 18) / 2 - 2, (COLS - 22) / 2 + 10);
     panel = new_panel(win);
     stdwinset(win);
     wattrset(win, STDATTR);
@@ -845,7 +889,7 @@ void removeaport(struct porttab **table)
     struct porttab *ptmp;
 
     selectport(table, &ptmp, &aborted);
-    
+
     if (!aborted) {
 	delport(table, ptmp);
 	saveportlist(*table);
@@ -855,7 +899,7 @@ void removeaport(struct porttab **table)
 void destroyporttab(struct porttab *table)
 {
     struct porttab *ptemp = table;
-    struct porttab *ctemp;
+    struct porttab *ctemp = NULL;
 
     if (ptemp != NULL)
 	ctemp = ptemp->next_entry;
