@@ -28,20 +28,23 @@ details.
 unsigned int bmaxy = 0;
 unsigned int imaxy = 0;
 
+void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
+		 unsigned int pktlen, char *message);
+
 void setlabels(WINDOW * win, int mode)
 {
-    wmove(win, 0, 42);
-    whline(win, ACS_HLINE, 23);
+    wmove(win, 0, 42 * COLS / 80);
+    whline(win, ACS_HLINE, 23 * COLS / 80);
 
     if (mode == 0) {
-	wmove(win, 0, 47);
+	wmove(win, 0, 47 * COLS / 80);
 	wprintw(win, " Packets ");
-	wmove(win, 0, 59);
+	wmove(win, 0, 59 * COLS / 80);
 	wprintw(win, " Bytes ");
     } else {
-	wmove(win, 0, 45);
+	wmove(win, 0, 45 * COLS / 80);
 	wprintw(win, " Pkt Size ");
-	wmove(win, 0, 56);
+	wmove(win, 0, 56 * COLS / 80);
 	wprintw(win, " Win Size ");
     }
 }
@@ -49,50 +52,48 @@ void setlabels(WINDOW * win, int mode)
 /*
  * The hash function for the TCP hash table
  */
- 
+
 unsigned int tcp_hash(unsigned long saddr, unsigned int sport,
-                      unsigned long daddr, unsigned int dport,
-                      char *ifname)
+		      unsigned long daddr, unsigned int dport,
+		      char *ifname)
 {
     int i;
     int ifsum = 0;
-    
+
     for (i = 0; i <= strlen(ifname) - 1; i++)
-        ifsum += ifname[i];
-    
-    return ((ifsum + (4 * saddr) + (3 * sport) + 
-            (2 * daddr) + dport) % ENTRIES_IN_HASH_TABLE);
+	ifsum += ifname[i];
+
+    return ((ifsum + (4 * saddr) + (3 * sport) +
+	     (2 * daddr) + dport) % ENTRIES_IN_HASH_TABLE);
 }
 
 void init_tcp_table(struct tcptable *table)
 {
     int i;
-    
-    table->bmaxy = LINES * 0.6;	             /* 60% of total screen */
+
+    table->bmaxy = LINES * 0.6;	/* 60% of total screen */
     table->imaxy = table->bmaxy - 2;
 
-    table->borderwin = newwin(table->bmaxy, 80, 1, 0);
+    table->borderwin = newwin(table->bmaxy, COLS, 1, 0);
     table->borderpanel = new_panel(table->borderwin);
 
     wattrset(table->borderwin, BOXATTR);
     box(table->borderwin, ACS_VLINE, ACS_HLINE);
     wmove(table->borderwin, 0, 1);
-    wprintw(table->borderwin, " Source ");
-    wmove(table->borderwin, 0, 23);
-    wprintw(table->borderwin, " Destination ");
+    wprintw(table->borderwin, " TCP Connections (Source Host:Port) ");
 
     setlabels(table->borderwin, 0);	/* initially use mode 0 */
 
-    wmove(table->borderwin, 0, 65);
+    wmove(table->borderwin, 0, 65 * COLS / 80);
     wprintw(table->borderwin, " Flags ");
-    wmove(table->borderwin, 0, 72);
+    wmove(table->borderwin, 0, 72 * COLS / 80);
     wprintw(table->borderwin, " Iface ");
     update_panels();
     doupdate();
 
     table->head = table->tail = NULL;
     table->firstvisible = table->lastvisible = NULL;
-    table->tcpscreen = newwin(table->imaxy, 78, 2, 1);
+    table->tcpscreen = newwin(table->imaxy, COLS - 2, 2, 1);
     table->tcppanel = new_panel(table->tcpscreen);
     table->closedentries = NULL;
     wattrset(table->tcpscreen, BOXATTR);
@@ -102,60 +103,58 @@ void init_tcp_table(struct tcptable *table)
 
     wtimeout(table->tcpscreen, -1);
     stdwinset(table->tcpscreen);
-    
+
     /* Initialize hash table with dummy header cells */
-    
+
     for (i = 0; i <= ENTRIES_IN_HASH_TABLE - 1; i++) {
-        table->hash_table[i] = NULL;
-        table->hash_tails[i] = NULL;
+	table->hash_table[i] = NULL;
+	table->hash_tails[i] = NULL;
     }
 }
 
 /*
  * Add a TCP entry to the hash table.
  */
- 
+
 int add_tcp_hash_entry(struct tcptable *table, struct tcptableent *entry)
 {
-    unsigned int hp;            /* hash position in table */
+    unsigned int hp;		/* hash position in table */
     struct tcp_hashentry *ptmp;
-    
+
     hp = tcp_hash(entry->saddr.s_addr, entry->sport,
-                  entry->daddr.s_addr, entry->dport, entry->ifname);
-    
+		  entry->daddr.s_addr, entry->dport, entry->ifname);
+
     ptmp = malloc(sizeof(struct tcp_hashentry));
     bzero(ptmp, sizeof(struct tcp_hashentry));
-    
-    if (ptmp == NULL)      
-        return 1;
-    
+
+    if (ptmp == NULL)
+	return 1;
+
     /*
      * Add backpointer from screen node to hash node for deletion later
      * (Actually point to its predecessor coz of the header cell).
      */
-     
+
     entry->hash_node = ptmp;
 
     /* 
      * Update hash node and add it to list.
      */
-     
+
     ptmp->tcpnode = entry;
     ptmp->hp = hp;
-    
+
     if (table->hash_table[hp] == NULL) {
-        ptmp->prev_entry = NULL;
-        table->hash_table[hp] = ptmp;
+	ptmp->prev_entry = NULL;
+	table->hash_table[hp] = ptmp;
     }
-    
     if (table->hash_tails[hp] != NULL) {
-        table->hash_tails[hp]->next_entry = ptmp;
-        ptmp->prev_entry = table->hash_tails[hp];
+	table->hash_tails[hp]->next_entry = ptmp;
+	ptmp->prev_entry = table->hash_tails[hp];
     }
-    
     table->hash_tails[hp] = ptmp;
     ptmp->next_entry = NULL;
-    
+
     return 0;
 }
 
@@ -167,24 +166,24 @@ void del_tcp_hash_node(struct tcptable *table, struct tcptableent *entry)
 {
     struct tcp_hashentry *ptmp;
 
-    ptmp = entry->hash_node;        /* ptmp now points to the target */
-    
+    ptmp = entry->hash_node;	/* ptmp now points to the target */
+
     /*
      * If the targeted node is the last entry, adjust the corresponding tail
      * pointer to the preceeding node;
      */
-     
+
     if (ptmp->next_entry == NULL)
-        table->hash_tails[ptmp->hp] = ptmp->prev_entry;
-        
+	table->hash_tails[ptmp->hp] = ptmp->prev_entry;
+
     if (ptmp->prev_entry != NULL)
-        ptmp->prev_entry->next_entry = ptmp->next_entry;
+	ptmp->prev_entry->next_entry = ptmp->next_entry;
     else
-        table->hash_table[ptmp->hp] = ptmp->next_entry;    
-    
+	table->hash_table[ptmp->hp] = ptmp->next_entry;
+
     if (ptmp->next_entry != NULL)
-        ptmp->next_entry->prev_entry = ptmp->prev_entry;
-        
+	ptmp->next_entry->prev_entry = ptmp->prev_entry;
+
     free(ptmp);
 }
 
@@ -194,7 +193,7 @@ void del_tcp_hash_node(struct tcptable *table, struct tcptableent *entry)
 
 struct tcptableent *addentry(struct tcptable *table,
 			     unsigned long int saddr,
- 			     unsigned long int daddr,
+			     unsigned long int daddr,
 			     unsigned int sport, unsigned int dport,
 			     int protocol, char *ifname, int rev_lookup,
 			     int rvnfd, int servnames, int *nomem)
@@ -265,13 +264,13 @@ struct tcptableent *addentry(struct tcptable *table,
 	table->closedentries = table->closedentries->next_entry;
 	free(ctemp);
 	new_entry->reused = new_entry->oth_connection->reused = 1;
-	
+
 	/*
 	 * Delete the old hash entries for this reallocated node;
 	 */
-        
-        del_tcp_hash_node(table, new_entry);
-        del_tcp_hash_node(table, new_entry->oth_connection);	
+
+	del_tcp_hash_node(table, new_entry);
+	del_tcp_hash_node(table, new_entry->oth_connection);
     }
 
     /* 
@@ -290,14 +289,14 @@ struct tcptableent *addentry(struct tcptable *table,
     new_entry->win = new_entry->psize = 0;
     new_entry->oth_connection->pcount = new_entry->oth_connection->bcount = 0;
     new_entry->oth_connection->win = new_entry->oth_connection->psize = 0;
-    
+
     /*
      * Store interface name
      */
-     
+
     strcpy(new_entry->ifname, ifname);
     strcpy(new_entry->oth_connection->ifname, ifname);
-    
+
     /*
      * Set raw port numbers
      */
@@ -317,8 +316,8 @@ struct tcptableent *addentry(struct tcptable *table,
      * Set port service names (where applicable)
      */
 
-    servlook(servnames, sport, IPPROTO_TCP, new_entry->s_sname, 5);
-    servlook(servnames, dport, IPPROTO_TCP, new_entry->d_sname, 5);
+    servlook(servnames, sport, IPPROTO_TCP, new_entry->s_sname, 10);
+    servlook(servnames, dport, IPPROTO_TCP, new_entry->d_sname, 10);
 
     strcpy(new_entry->oth_connection->s_sname, new_entry->d_sname);
     strcpy(new_entry->oth_connection->d_sname, new_entry->s_sname);
@@ -344,10 +343,10 @@ struct tcptableent *addentry(struct tcptable *table,
     /*
      * Add entries to hash table
      */
-    
+
     *nomem = add_tcp_hash_entry(table, new_entry);
     *nomem = add_tcp_hash_entry(table, new_entry->oth_connection);
-    
+
     return new_entry;
 }
 
@@ -372,31 +371,29 @@ void addtoclosedlist(struct tcptable *table, struct tcptableent *entry,
 
 struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
 			     unsigned long daddr, unsigned int sport,
-			     unsigned int dport, char *ifname, int *isclosed,
+			     unsigned int dport, char *ifname,
 			     unsigned int tm, int logging, FILE * logfile,
 			     int *nomem)
 {
     struct tcp_hashentry *hashptr;
     unsigned int hp;
-    
-    char msgstring[120];
+
+    char msgstring[240];
 
     int hastimeouts = 0;
 
     time_t now;
 
     if (table->head == NULL) {
-	*isclosed = 0;
 	return 0;
     }
-
     /*
      * Determine hash table index for this set of addresses and ports
      */
-     
+
     hp = tcp_hash(saddr, sport, daddr, dport, ifname);
     hashptr = table->hash_table[hp];
-     
+
     while (hashptr != NULL) {
 	if ((hashptr->tcpnode->saddr.s_addr == saddr) &&
 	    (hashptr->tcpnode->daddr.s_addr == daddr) &&
@@ -418,10 +415,10 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
 		hastimeouts = 1;
 
 	    if (logging) {
-		sprintf(msgstring, "TCP: Connection %s:%s <-> %s:%s idle for %u minutes, marking for timeout",
-			hashptr->tcpnode->s_fqdn, hashptr->tcpnode->s_sname,
-			hashptr->tcpnode->d_fqdn, hashptr->tcpnode->d_sname,
-			tm);
+		sprintf(msgstring, "TCP; Connection %s:%s <-> %s:%s idle for %u minutes, marking for timeout, %lu packets, %lu bytes",
+		     hashptr->tcpnode->s_fqdn, hashptr->tcpnode->s_sname,
+		     hashptr->tcpnode->d_fqdn, hashptr->tcpnode->d_sname,
+		 tm, hashptr->tcpnode->pcount, hashptr->tcpnode->bcount);
 		writelog(logging, logfile, msgstring);
 	    }
 	}
@@ -429,17 +426,15 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
     }
 
     if (hashptr != NULL) {	/* needed to avoid SIGSEGV */
-	if (((hashptr->tcpnode->finsent == 2) || (hashptr->tcpnode->stat & FLAG_RST)) &&
-	    ((hashptr->tcpnode->oth_connection->finsent == 2) ||
-	     (hashptr->tcpnode->oth_connection->stat & FLAG_RST))) {
-	    *isclosed = 1;
+	if ((((hashptr->tcpnode->finsent == 2) &&
+	      (hashptr->tcpnode->oth_connection->finsent == 2))) ||
+	    (((hashptr->tcpnode->stat & FLAG_RST) ||
+	      (hashptr->tcpnode->oth_connection->stat & FLAG_RST)))) {
 	    return NULL;
 	} else {
-	    *isclosed = 0;
 	    return hashptr->tcpnode;
 	}
     } else {
-	*isclosed = hastimeouts;
 	return NULL;
     }
 }
@@ -515,12 +510,8 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	    tableentry->oth_connection->finsent = 2;
 
 	    if (logging) {
-		sprintf(msgstring, "TCP: %s:%s to %s:%s: FIN acknowledged. %lu packets, %lu bytes sent",
-			tableentry->s_fqdn, tableentry->s_sname,
-			tableentry->d_fqdn, tableentry->d_sname,
-			tableentry->pcount, tableentry->bcount);
-
-		writelog(logging, logfile, msgstring);
+		writetcplog(logging, logfile, tableentry, tableentry->psize,
+			    "FIN acknowleged");
 	    }
 	}
     }
@@ -561,12 +552,10 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	    tableentry->finack = ntohl(transpacket->ack_seq);
 	}
 	if (logging) {
-	    sprintf(msgstring, "TCP: %s:%s to %s:%s: FIN sent pkt size=%u bytes",
-		    tableentry->s_fqdn, tableentry->s_sname,
-		    tableentry->d_fqdn, tableentry->d_sname,
-		    tableentry->psize);
+	    sprintf(msgstring, "FIN sent; %lu packets, %lu bytes",
+		    tableentry->pcount, tableentry->bcount);
 
-	    writelog(logging, logfile, msgstring);
+	    writetcplog(logging, logfile, tableentry, tableentry->psize, msgstring);
 	}
     }
     if (transpacket->rst) {
@@ -575,10 +564,11 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	    addtoclosedlist(table, tableentry, nomem);
 
 	if (logging) {
-	    sprintf(msgstring, "TCP: %s:%s to %s:%s: Connection reset",
-		    tableentry->s_fqdn, tableentry->s_sname,
-		    tableentry->d_fqdn, tableentry->d_sname);
-	    writelog(logging, logfile, msgstring);
+	    sprintf(msgstring, "Connection reset; %lu packets, %lu bytes; opposite direction %lu packets, %lu bytes",
+		    tableentry->pcount, tableentry->bcount,
+		    tableentry->oth_connection->pcount,
+		    tableentry->oth_connection->bcount);
+	    writetcplog(logging, logfile, tableentry, tableentry->psize, msgstring);
 	}
     }
     if (transpacket->psh)
@@ -588,14 +578,14 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	tableentry->stat |= FLAG_URG;
 
     tableentry->lastupdate = tableentry->oth_connection->lastupdate = time(NULL);
-	/*
-	 * Shall we add this entry to the closed entry list?  If both
-	 * directions have their state indicators set to 2, that's it.
-	 */
+    /*
+     * Shall we add this entry to the closed entry list?  If both
+     * directions have their state indicators set to 2, that's it.
+     */
 
     if ((!tableentry->inclosed) &&
-       (((tableentry->finsent == 2) && ((tableentry->oth_connection->finsent == 1) || (tableentry->oth_connection->finsent == 2))) ||
-       ((tableentry->oth_connection->finsent == 2) && ((tableentry->finsent == 1) || (tableentry->finsent == 2)))))
+	(((tableentry->finsent == 2) && ((tableentry->oth_connection->finsent == 1) || (tableentry->oth_connection->finsent == 2))) ||
+	 ((tableentry->oth_connection->finsent == 2) && ((tableentry->finsent == 1) || (tableentry->finsent == 2)))))
 	addtoclosedlist(table, tableentry, nomem);
 
 }
@@ -635,6 +625,7 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 {
     char stat[7] = "";
     unsigned int target_row;
+    char sp_buf[255];
 
     if ((tableentry->index < screen_idx) ||
 	(tableentry->index > screen_idx + (table->imaxy - 1)))
@@ -648,7 +639,8 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
     wmove(table->tcpscreen, target_row, 2);
     if (tableentry->reused) {
 	scrollok(table->tcpscreen, 0);
-	wprintw(table->tcpscreen, "%76c", 32);
+	sprintf(sp_buf, "%%%dc", COLS - 4);
+	wprintw(table->tcpscreen, sp_buf, ' ');
 	scrollok(table->tcpscreen, 1);
 	tableentry->reused = 0;
 	wmove(table->tcpscreen, target_row, 1);
@@ -660,13 +652,11 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 
     /* proceed with the actual entry */
 
+    sprintf(sp_buf, "%%.%ds:%%.%ds", 32 * COLS / 80, 10);
     wattrset(table->tcpscreen, STDATTR);
 
     wmove(table->tcpscreen, target_row, 1);
-    wprintw(table->tcpscreen, "%s:%s", tableentry->s_fqdn, tableentry->s_sname);
-
-    wmove(table->tcpscreen, target_row, 23);
-    wprintw(table->tcpscreen, "%s:%s", tableentry->d_fqdn, tableentry->d_sname);
+    wprintw(table->tcpscreen, sp_buf, tableentry->s_fqdn, tableentry->s_sname);
 
     wattrset(table->tcpscreen, HIGHATTR);
 
@@ -676,14 +666,19 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
      */
 
     if (mode == 0) {
-	wmove(table->tcpscreen, target_row, 45);
+	wmove(table->tcpscreen, target_row, 47 * COLS / 80 - 2);
 	if (tableentry->partial)
-	    wprintw(table->tcpscreen, ">%8u %9u", tableentry->pcount, tableentry->bcount);
+	    wprintw(table->tcpscreen, ">");
 	else
-	    wprintw(table->tcpscreen, " %8u %9u", tableentry->pcount, tableentry->bcount);
+	    wprintw(table->tcpscreen, "=");
+	wprintw(table->tcpscreen, "%8u  ", tableentry->pcount);
+	wmove(table->tcpscreen, target_row, 59 * COLS / 80 - 4);
+	wprintw(table->tcpscreen, "%9u  ", tableentry->bcount);
     } else {
-	wmove(table->tcpscreen, target_row, 48);
-	wprintw(table->tcpscreen, "%5u  %9u ", tableentry->psize, tableentry->win);
+	wmove(table->tcpscreen, target_row, 45 * COLS / 80 + 3);
+	wprintw(table->tcpscreen, "%5u  ", tableentry->psize);
+	wmove(table->tcpscreen, target_row, 56 * COLS / 80 - 1);
+	wprintw(table->tcpscreen, "%9u  ", tableentry->win);
     }
 
     wattrset(table->tcpscreen, STDATTR);
@@ -718,9 +713,9 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 	strcat(stat, " ");
     }
 
-    wmove(table->tcpscreen, target_row, 65);
+    wmove(table->tcpscreen, target_row, 65 * COLS / 80);
     wprintw(table->tcpscreen, "%s", stat);
-    wmove(table->tcpscreen, target_row, 72);
+    wmove(table->tcpscreen, target_row, 72 * COLS / 80);
     wprintw(table->tcpscreen, "%s", tableentry->ifname);
 }
 
@@ -754,13 +749,13 @@ void destroytcptable(struct tcptable *table)
     struct closedlist *closedtemp_next;
     struct tcp_hashentry *hashtemp;
     struct tcp_hashentry *hashtemp_next;
-    
+
     unsigned int i;
 
     /*
      * Destroy main TCP table
      */
-     
+
     if (table->head != NULL) {
 	ctemp = table->head;
 	c_next_entry = table->head->next_entry;
@@ -773,11 +768,10 @@ void destroytcptable(struct tcptable *table)
 		c_next_entry = c_next_entry->next_entry;
 	}
     }
- 
     /*
      * Destroy list of closed entries
      */
-     
+
     if (table->closedentries != NULL) {
 	closedtemp = table->closedentries;
 	closedtemp_next = table->closedentries->next_entry;
@@ -789,24 +783,22 @@ void destroytcptable(struct tcptable *table)
 		closedtemp_next = closedtemp_next->next_entry;
 	}
     }
-    
     /*
      * Destroy hash table
      */
-     
-    for (i = 0; i <= ENTRIES_IN_HASH_TABLE - 1; i++) {
-        if (table->hash_table[i] != NULL) {
-            hashtemp = table->hash_table[i];
-            hashtemp_next = table->hash_table[i]->next_entry;
-           
-            while (hashtemp != NULL) {
-                free(hashtemp);
-                hashtemp = hashtemp_next;
-               
-                if (hashtemp_next != NULL)
-                    hashtemp_next = hashtemp_next->next_entry;
-            }
-        }
-    }     
-}
 
+    for (i = 0; i <= ENTRIES_IN_HASH_TABLE - 1; i++) {
+	if (table->hash_table[i] != NULL) {
+	    hashtemp = table->hash_table[i];
+	    hashtemp_next = table->hash_table[i]->next_entry;
+
+	    while (hashtemp != NULL) {
+		free(hashtemp);
+		hashtemp = hashtemp_next;
+
+		if (hashtemp_next != NULL)
+		    hashtemp_next = hashtemp_next->next_entry;
+	    }
+	}
+    }
+}
