@@ -40,14 +40,14 @@
 struct tcptableent {
     struct in_addr saddr;
     struct in_addr daddr;
-    char s_fqdn[20];		/* fully-qualified domain names */
-    char d_fqdn[20];
+    char s_fqdn[45];		/* fully-qualified domain names */
+    char d_fqdn[45];
     int s_fstat;
     int d_fstat;        
     unsigned int sport;
     unsigned int dport;
-    char s_sname[7];           /* Service names, maxlen=6 */
-    char d_sname[7];
+    char s_sname[11];           /* Service names, maxlen=10 */
+    char d_sname[11];
     unsigned int protocol;
     unsigned long pcount;	/* packet count */
     unsigned long bcount;	/* byte count */
@@ -115,7 +115,7 @@ struct tcptableent *addentry(struct tcptable *table,
 struct tcptableent *in_table(struct tcptable *table,
 			     unsigned long saddr, unsigned long daddr,
 			     unsigned int sport, unsigned int dport,
-			     char *ifname, int *isclosed, unsigned int tm,
+			     char *ifname, unsigned int tm,
 			     int logging, FILE * logfile, int *nomem);
 
 void updateentry(struct tcptable *table, struct tcptableent *tableentry,
