@@ -65,7 +65,7 @@ void gethostparams(struct hostparams *hp, int *aborted)
     WINDOW *dlgwin;
     PANEL *dlgpanel;
 
-    dlgwin = newwin(12, 80, 7, 0);
+    dlgwin = newwin(12, 80, (LINES - 12) / 2, (COLS - 80) / 2);
     dlgpanel = new_panel(dlgwin);
 
     wattrset(dlgwin, BOXATTR);
@@ -93,7 +93,7 @@ void gethostparams(struct hostparams *hp, int *aborted)
     update_panels();
     doupdate();
 
-    initfields(&fieldlist, 8, 52, 8, 27);
+    initfields(&fieldlist, 8, 52, (LINES - 12) / 2 + 1, (COLS - 80) / 2 + 27);
 
     addfield(&fieldlist, 25, 1, 0, "");
     addfield(&fieldlist, 25, 3, 0, "");
@@ -129,21 +129,21 @@ void gethostparams(struct hostparams *hp, int *aborted)
 int mark_filter_change(int protocol)
 {
     int resp;
-    
+
     if (protocol == F_TCP) {
-        if (!facility_active(TCPFLTIDFILE))
-            mark_facility(TCPFLTIDFILE, "TCP filter change");
-        else {
-            errbox("TCP filter data file in use; try again later", ANYKEY_MSG, &resp);
-            return 0;
-        }
+	if (!facility_active(TCPFLTIDFILE))
+	    mark_facility(TCPFLTIDFILE, "TCP filter change");
+	else {
+	    errbox("TCP filter data file in use; try again later", ANYKEY_MSG, &resp);
+	    return 0;
+	}
     } else {
-        if (!facility_active(UDPFLTIDFILE))
-            mark_facility(UDPFLTIDFILE, "UDP filter change");
-        else {
-            errbox("UDP filter data file in use; try again later", ANYKEY_MSG, &resp);
-            return 0;
-        }
+	if (!facility_active(UDPFLTIDFILE))
+	    mark_facility(UDPFLTIDFILE, "UDP filter change");
+	else {
+	    errbox("UDP filter data file in use; try again later", ANYKEY_MSG, &resp);
+	    return 0;
+	}
     }
     return 1;
 }
@@ -151,9 +151,9 @@ int mark_filter_change(int protocol)
 void clear_flt_tag(int protocol)
 {
     if (protocol == F_TCP)
-        unlink(TCPFLTIDFILE);
+	unlink(TCPFLTIDFILE);
     else
-        unlink(UDPFLTIDFILE);
+	unlink(UDPFLTIDFILE);
 }
 
 void definefilter(int protocol, int *aborted)
@@ -167,6 +167,8 @@ void definefilter(int protocol, int *aborted)
     int pfd;
     int bw;
     int resp;
+    
+    int dlgwintop;
 
     WINDOW *dlgwin;
     PANEL *dlgpanel;
@@ -174,11 +176,12 @@ void definefilter(int protocol, int *aborted)
     /*
      * Lock facility
      */
-     
+
     if (!mark_filter_change(protocol))
-        return;
-        
-    dlgwin = newwin(7, 42, 9, 9);
+	return;
+    
+    dlgwintop = (LINES - 9) / 2;
+    dlgwin = newwin(7, 42, dlgwintop, (COLS - 42) / 2 - 10);
     dlgpanel = new_panel(dlgwin);
     wattrset(dlgwin, BOXATTR);
     colorwin(dlgwin);
@@ -191,7 +194,7 @@ void definefilter(int protocol, int *aborted)
     update_panels();
     doupdate();
 
-    initfields(&descfield, 1, 35, 12, 11);
+    initfields(&descfield, 1, 35, dlgwintop + 3, (COLS - 42) / 2 - 8);
     addfield(&descfield, 33, 0, 0, "");
     fillfields(&descfield, aborted);
     strcpy(ffile.desc, descfield.list->buf);
@@ -202,10 +205,9 @@ void definefilter(int protocol, int *aborted)
     doupdate();
 
     if (*aborted) {
-        clear_flt_tag(protocol);
+	clear_flt_tag(protocol);
 	return;
     }
-
     genname(time((time_t *) NULL), fntemp);
     strcat(ffilename, fntemp);
 
@@ -213,7 +215,7 @@ void definefilter(int protocol, int *aborted)
     if (pfd < 0) {
 	errbox("Cannot create TCP/UDP filter record file", ANYKEY_MSG, &resp);
 	*aborted = 1;
-        clear_flt_tag(protocol);
+	clear_flt_tag(protocol);
 	return;
     }
     do {
@@ -224,7 +226,7 @@ void definefilter(int protocol, int *aborted)
 	    if (bw < 0) {
 		errbox("Unable to write TCP/UDP filter record", ANYKEY_MSG, &resp);
 		close(pfd);
-	        clear_flt_tag(protocol);
+		clear_flt_tag(protocol);
 		return;
 	    }
 	}
@@ -239,7 +241,7 @@ void definefilter(int protocol, int *aborted)
 
     if (pfd < 0) {
 	listfileerr(1);
-        clear_flt_tag(protocol);
+	clear_flt_tag(protocol);
 	return;
     }
     strcpy(ffile.filename, fntemp);
@@ -357,12 +359,12 @@ void pickafilter(struct ffnode *ffiles,
     WINDOW *selectwin;
     PANEL *selectpanel;
 
-    borderwin = newwin(10, 60, 6, 8);
+    borderwin = newwin(10, 60, (LINES - 10) / 2 - 2, (COLS - 60) / 2 - 2);
     borderpanel = new_panel(borderwin);
     wattrset(borderwin, BOXATTR);
     box(borderwin, ACS_VLINE, ACS_HLINE);
 
-    selectwin = newwin(8, 58, 7, 9);
+    selectwin = newwin(8, 58, (LINES - 8) / 2 - 2, (COLS - 58) / 2 - 2);
     selectpanel = new_panel(selectwin);
     wattrset(selectwin, STDATTR);
     colorwin(selectwin);
@@ -473,8 +475,9 @@ void loadfilter(char *filename, struct filterlist *fl)
 	    fe->next_entry = NULL;
 	    fl->tail = fe;
 	    idx++;
-	} else
+	} else {
 	    free(fe);
+	}
     } while (br > 0);
 
     if (br == 0)
@@ -599,7 +602,7 @@ void delfilter(unsigned int protocol, int *aborted)
     int bw;
 
     if (!mark_filter_change(protocol))
-        return;
+	return;
 
     if (loadfilterlist(protocol, &fltlist) == 1) {
 	listfileerr(1);
@@ -613,7 +616,6 @@ void delfilter(unsigned int protocol, int *aborted)
 	clear_flt_tag(protocol);
 	return;
     }
-
     strcat(fntemp, fltfile->ffe.filename);
     unlink(fntemp);
 
@@ -655,13 +657,13 @@ void delfilter(unsigned int protocol, int *aborted)
     }
 
     close(fd);
-    
+
     clear_flt_tag(protocol);
 }
 
 void makefiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 7, 31, 10, 30);
+    initmenu(menu, 7, 31, (LINES - 7) / 2 + 3, (COLS - 31) / 2 + 15);
     additem(menu, " ^D^efine new filter...", "Defines a new set of filter parameters");
     additem(menu, " ^A^pply filter...", "Applies filter to be used for TCP display");
     additem(menu, " Detac^h^ filter", "Removes the currently applied filter");
@@ -671,7 +673,7 @@ void makefiltermenu(struct MENU *menu)
 
 void makeudpfiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 8, 31, 10, 30);
+    initmenu(menu, 8, 31, (LINES - 11) / 2, (COLS - 30) / 2 - 21);
     additem(menu, " Show ^a^ll UDP packets", "Permits display of all UDP packets");
     additem(menu, " Show ^n^o UDP packets", "Omits all UDP packets from display");
     additem(menu, " ^D^efine custom UDP filter...", "Defines a custom UDP filter");
